<?php

require( 'common/admin.php' );

class Meow_WPLR_Sync_Admin extends MeowApps_Admin {

	protected $hide_ads;
	protected $meow_admin;
	protected $wplr;

	public function __construct( $prefix, $mainfile, $domain ) {
		parent::__construct( $prefix, $mainfile, $domain );
		if ( is_admin() ) {
			global $wplr;
			$this->wplr = $wplr;
			$this->wplr->check_db();
			add_action( 'admin_menu', array( $this, 'admin_menu' ) );
			add_action( 'admin_notices', array( $this, 'admin_notices' ) );
			add_filter( 'media_row_actions', array( $this, 'media_row_actions' ), 10, 2 );
			add_action( 'wp_ajax_wplrsync_link', array( $wplr, 'wplrsync_link' ) );
			add_action( 'wp_ajax_wplrsync_unlink', array( $wplr, 'wplrsync_unlink' ) );
			add_action( 'wp_ajax_wplrsync_clean', array( $this, 'wplrsync_clean' ) );
			add_action( 'wp_ajax_wplrsync_extensions_reset', array( $this, 'wplrsync_extensions_reset' ) );
			add_action( 'wp_ajax_wplrsync_extensions_init', array( $this, 'wplrsync_extensions_init' ) );
			add_action( 'wp_ajax_wplrsync_extensions_query', array( $this, 'wplrsync_extensions_query' ) );
			$this->hide_ads = get_option( 'wplr_hide_ads', false );
			if ( get_option( 'wplr_library_show_hierarchy' ) ) {
				include_once( 'lrsync_library.php' );
				$wplr_medialib = new Meow_WPLR_Sync_Media_Library();
			}
		}
	}

	function common_url( $file ) {
		return plugin_dir_url( __FILE__ ) . ( '\/common\/' . $file );
	}

	function admin_notices() {
		// W3 Total Cache
		if ( defined( 'W3TC' ) && function_exists( 'w3tc_config' ) ) {
			$config = w3tc_config();
			if ( !empty( $config ) && $config->get_string( 'dbcache.enabled' ) ) {
				?>
			    <div class="notice notice-error is-dismissible">
			        <p><?php _e( 'WP/LR Sync will not work properly because you are using W3 Total Cache with the Database Cached enabled. For some reason, this option breaks the syncing process.', 'wplr-sync' ); ?></p>
			    </div>
		    <?php
			}
		}

		$permastruct = get_option( 'permalink_structure' );
		if ( empty( $permastruct ) ) {
		?>
	    <div class="notice notice-error is-dismissible">
	        <p><?php _e( 'WP/LR Sync will not work properly if your permalinks are set up on "Plain". Please pick a dynamic structure for your permalinks (Settings > Permalinks).', 'wplr-sync' ); ?></p>
	    </div>
    <?php
		}
	}

	function media_row_actions( $actions, $post ) {
		global $current_screen;
		if ( 'upload' != $current_screen->id )
		    return $actions;
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$res = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $table_name WHERE wp_id = %d", $post->ID ) );
		if ( !empty( $res ) && isset( $actions['trash'] ) )
			$actions['trash'] = "Trash";
		return $actions;
	}

	function admin_menu() {
		$hide_extensions = get_option( 'wplr_hide_extensions' ) && !get_option( 'wplr_debugging_enabled' );

		// Standard menu
		add_menu_page( 'WP/LR', 'WP/LR Sync', 'manage_options', 'wplr-main-menu', array( $this,
			'admin_settings' ), 'dashicons-camera', 82 );

		// Settings
		add_submenu_page( 'wplr-main-menu', __( 'Settings', 'wplr-sync' ), __( 'Settings', 'wplr-sync' ), 'manage_options',
			'wplr-main-menu', array( $this, 'admin_settings' ) );
		add_settings_section( 'wplr_settings', null, array( $this, 'admin_settings_intro' ),
			'wplr-settings-menu' );
		add_settings_field( 'wplr_use_taken_date', __( 'Meta Date', 'wplr-sync' ), array( $this, 'admin_takendate_callback' ),
			'wplr-settings-menu', 'wplr_settings', array( __( 'Use Photo Taken Date', 'wplr-sync' ) ) );

		add_settings_field( 'wplr_enable_keywords', __( 'Enable Keywords', 'wplr-sync' ),
			array( $this, 'admin_enable_keywords_callback' ),
			'wplr-settings-menu', 'wplr_settings',
			array( __( 'Create the Keywords taxonomy', 'wplr-sync' ) ) );
		add_settings_field( 'wplr_sync_keywords', __( 'Sync Keywords', 'wplr-sync' ), array( $this, 'admin_sync_keywords_callback' ),
			'wplr-settings-menu', 'wplr_settings', array( __( 'Sync your keywords in LR with a WP taxonomy.', 'wplr-sync' ) ) );

			add_settings_field( 'wplr_library_show_hierarchy',
				__( 'Show Folders & Collections in Media Library', 'wplr-sync' ),
				array( $this, 'admin_library_show_hierarchy_callback' ),
				'wplr-settings-menu', 'wplr_settings', array( "Enable" ) );

		add_settings_field( 'wplr_filename_accents', __( 'Skip Sanitization', 'wplr-sync' ),
			array( $this, 'admin_filename_accents_callback' ),
			'wplr-settings-menu', 'wplr_settings', array( "Enable" ) );
		add_settings_field( 'wplr_debugging_enabled', __( 'Debugging Tools', 'wplr-sync' ),
			array( $this, 'admin_debugging_callback' ),
			'wplr-settings-menu', 'wplr_settings', array( "Enable" ) );
		add_settings_field( 'wplr_debuglogs', __( 'Advanced Logs', 'wplr-sync' ),
			array( $this, 'admin_debuglogs_callback' ),
			'wplr-settings-menu', 'wplr_settings', array( "Enable" ) );

		add_settings_field( 'wplr_public_api', __( 'JSON API<br />(Mobile Apps)', 'wplr-sync' ),
			array( $this, 'admin_public_api_callback' ),
			'wplr-settings-menu', 'wplr_settings', array( "Enable" ) );

		register_setting( 'wplr-settings', 'wplr_use_taken_date' );
		register_setting( 'wplr-settings', 'wplr_enable_keywords' );
		register_setting( 'wplr-settings', 'wplr_sync_keywords' );
		register_setting( 'wplr-settings', 'wplr_filename_accents' );
		register_setting( 'wplr-settings', 'wplr_library_show_hierarchy' );
		register_setting( 'wplr-settings', 'wplr_debuglogs' );
		register_setting( 'wplr-settings', 'wplr_debugging_enabled' );
		register_setting( 'wplr-settings', 'wplr_public_api' );

		// Tools
		add_submenu_page( 'wplr-main-menu',
			__( 'Collections & Keywords', 'wplr-sync' ),
			__( 'Collections & Keywords', 'wplr-sync' ), 'manage_options',
			'wplr-collections-tags-menu', array( $this, 'admin_collections_keywords' ) );

		// Tools
		add_submenu_page( 'wplr-main-menu',
			__( 'Media Tools', 'wplr-sync' ),
			__( 'Media Tools', 'wplr-sync' ), 'manage_options',
			'wplr-tools-menu', array( $this, 'admin_tools' ) );

		// Debug
		if ( get_option( 'wplr_debugging_enabled' ) )
			add_submenu_page( 'wplr-main-menu',
				__( 'Debugging Tools', 'wplr-sync' ),
				__( 'Debugging Tools', 'wplr-sync' ), 'manage_options',
				'wplrsync-debug-menu', array( $this, 'admin_debug' ) );

		// Extensions
		if ( !$hide_extensions ) {
			add_submenu_page( 'wplr-main-menu',
				__( 'Extensions', 'wplr-sync' ),
				__( 'Extensions', 'wplr-sync' ), 'manage_options',
				'wplr-extensions-menu', array( $this, 'admin_extensions' ) );
			add_settings_field( 'wplr_plugins', null, array( $this, 'admin_extensions_callback' ),
				'wplr-extensions-menu', 'wplr_extensions' );
			register_setting( 'wplr-extensions', 'wplr_plugins' );
		}
	}

	function admin_settings() {
		$wplr_token = get_transient( 'wplr_auth_token' );
		$home_url = function_exists( 'pll_home_url' ) ? pll_home_url() : get_home_url();
		if ( $wplr_token === false ) {
			$wplr_token = str_replace ( '|', '#', wp_generate_password( 12, false, false ) );
			set_transient( 'wplr_auth_token', $wplr_token, YEAR_IN_SECONDS );
		}
		$qr = $home_url . '|' . $wplr_token;
		?>
		<div class="wrap">
			<?php echo $this->display_title( __( 'Settings', 'wplr-sync' ) . " | WP/LR Sync" );  ?>
			<div class="meow-section meow-group">
				<div class="meow-col meow-span_2_of_3">
					<div class="meow-box">
						<h3>Settings</h3>
						<div class="inside">
							<form method="post" action="options.php">
									<?php settings_fields( 'wplr-settings' ); ?>
							    <?php do_settings_sections( 'wplr-settings-menu' ); ?>
							    <?php submit_button(); ?>
							</form>
						</div>
					</div>
				</div>

				<?php if ( get_option( 'wplr_public_api', false ) ): ?>
				<div class="meow-col meow-span_1_of_3">
					<div class="meow-box">
						<h3>QR Code</h3>
						<center>
							<img style='margin: 20px;' width='200'
								src="http://chart.apis.google.com/chart?cht=qr&chs=400x400&chl=<?php echo $qr;?>&chld=H|0" />
						</center>
					</div>
				</div>
				<?php else: delete_transient( 'wplr_auth_token' ); ?>
				<?php endif; ?>

			</div>
		</div>
		<?php
	}

	function admin_extensions() {
		?>
		<div class="wrap">
			<?php echo $this->display_title( __( 'Extensions', 'wplr-sync' ) . " | WP/LR Sync" );  ?>
			<div class="meow-section meow-group">
					<div class="meow-box">
						<div class="inside">
							<?php $this->admin_extension_intro(); ?>
						</div>
					</div>
			</div>

			<div class="meow-section meow-group">

				<div class="meow-col meow-span_1_of_2">
					<div class="meow-box">
						<h3><?php echo __( 'Extensions', 'wplr-sync' ); ?></h3>
						<div class="inside">
							<form method="post" action="options.php">
									<?php settings_fields( 'wplr-extensions' ); ?>
									<?php $this->admin_extensions_callback(); ?>
							    <?php submit_button(); ?>
							</form>
						</div>
					</div>
				</div>

				<div class="meow-col meow-span_1_of_2">
					<div class="meow-box">
						<h3><?php echo __( 'Actions', 'wplr-sync' ); ?></h3>
						<div class="inside">
							<?php $this->admin_maintenance_callback(); ?>
						</div>
					</div>
				</div>

			</div>

		</div>
		<?php
	}

	/*
		OPTIONS
	*/

	function admin_settings_intro() {
		$this->display_message_about_loaded_extensions();
	}

	function display_message_about_loaded_extensions() {
		$extensions = apply_filters( 'wplr_extensions', array() );
		$html = "<p>";
		if ( empty( $extensions ) )
			$html .= "<div class='updated'><p>" .
				__( 'No extensions have been loaded (and that is fine).', 'wplr-sync' ) . "</p></div>";
		else if ( count( $extensions ) == 1 ) {
			$html .= "<div class='updated'><p>" .
				sprintf( __( 'The extension for <b>%s</b> is loaded.', 'wplr-sync' ), $extensions[0] ) .
				"</p></div>";
		}
		else {
			$html .= "<div class='updated'><p>" .
				sprintf( __( 'The extensions for <b>%s</b> are loaded.', 'wplr-sync' ), implode( ', ', $extensions ) ) .
				"</p></div>";
		}
		$html .= "</p>";
		echo $html;
	}

	function admin_extension_intro() {
		echo '<p>';
		_e( 'The plugin synchronizes your collections and collection sets from Lightroom. They are stored in WordPress internally but not used nor displayed initially. Extensions are required to use those collections. You can learn more about it here: <a target="_blank" href="http://meowapps.com/wplr-sync/wplr-extensions/">WP/LR Extensions</a>.', 'wplr-sync' );
		echo '</p>';
		$this->display_message_about_loaded_extensions();
	}

	function sort_by_order( $a, $b ) {
		return $a['order'] > $b['order'];
	}

	function admin_extensions_callback() {
		$options = get_option( 'wplr_plugins', array() );
		$plugins = array();
		$dir = trailingslashit( plugin_dir_path( __FILE__ ) ) . trailingslashit( 'extensions' );
		foreach ( glob( $dir . "*.*" ) as $filename ) {
	    $content = file_get_contents( $filename );
			preg_match( "/Plugin Name: (.*)/", $content, $name );
			preg_match( "/Description: (.*)/", $content, $desc );
			preg_match( "/Plugin Key: (.*)/", $content, $order );
			if ( count( $name ) > 1 ) {
				$info = pathinfo( $filename );
				$plugins[$info['basename']] = array(
					'name' => $name[1],
					'desc' => count( $desc ) > 1 ? $desc[1] : "",
					'order' => empty( $order ) ? 1000 : $order[1]
				 );
			}
		}
		$html = '';
		$c = 0;
		uasort( $plugins, array( $this, 'sort_by_order' ) );

		foreach ( $plugins as $key => $value ) {
			$checked = ( !empty( $options ) && in_array( $key, $options ) ) ? 'checked="checked"' : '';
			if ( $c++ > 0 )
				$html .= "<br />";
    	$html .= sprintf( '<input type="checkbox" id="%1$s[%2$s]" name="%1$s[]" value="%2$s" %3$s />', "wplr_plugins", $key, $checked );
    	$html .= sprintf( '<label for="%1$s[%3$s]"> %2$s</label><br>', "wplr_plugins", $value['name'], $key );
			$html .= '<span class="description">' . $value['desc'] . '</span><br />';
		}
		echo $html;
	}

	function wplrsync_clean() {
		global $wpdb;
		$tbl_m = $wpdb->prefix . 'lrsync';
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$tbl_c = $wpdb->prefix . 'lrsync_collections';
		$tbl_mt = $wpdb->prefix . 'lrsync_meta';
		try {
			do_action( 'wplr_clean' );
			$wpdb->query( "DELETE FROM $tbl_m WHERE wp_id NOT IN (SELECT ID FROM $wpdb->posts WHERE post_type = 'attachment')" );
			$wpdb->query( "DELETE FROM $tbl_r WHERE wp_id NOT IN (SELECT ID FROM $wpdb->posts WHERE post_type = 'attachment')" );
			$wpdb->query( "DELETE FROM $tbl_r WHERE wp_col_id NOT IN (SELECT wp_col_id FROM $tbl_c)" );
			$wpdb->query( "DELETE FROM $tbl_mt WHERE name = 'media_tag' AND id NOT IN (SELECT ID FROM $wpdb->posts WHERE post_type = 'attachment')" );
			echo json_encode( array( 'success' => true ) );
		}
		catch ( Exception $e ) {
			error_log( print_r( $e, true ) );
			echo json_encode( array( 'success' => false, 'message' => "An exception was caught and written in the PHP error logs." ) );
		}
		die();
	}

	function wplrsync_extensions_reset() {
		try {
			do_action( 'wplr_reset' );
			echo json_encode( array( 'success' => true ) );
		}
		catch ( Exception $e ) {
			error_log( print_r( $e, true ) );
			echo json_encode( array( 'success' => false, 'message' => "An exception was caught and written in the PHP error logs." ) );
		}
		die();
	}

	function wplrsync_extensions_query() {
		try {
			global $wplr;
			$task = $_POST['task'];
			if ( $task['action'] == 'add_collection' ) {
				if ( $task['is_folder'] )
					do_action( 'wplr_create_folder', (int)$task['wp_col_id'], (int)$task['wp_folder_id'], array( 'name' => $task['name'] ) );
				else
					do_action( 'wplr_create_collection', (int)$task['wp_col_id'], (int)$task['wp_folder_id'], array( 'name' => $task['name'] ) );
			}
			else if ( $task['action'] == 'remove_collection' ) {
				if ( $task['is_folder'] )
					do_action( 'wplr_remove_folder', (int)$task['wp_col_id'] );
				else
					do_action( 'wplr_remove_collection', (int)$task['wp_col_id'] );
			}
			else if ( $task['action'] == 'add_media' )
				do_action( 'wplr_add_media_to_collection', (int)$task['wp_id'], (int)$task['wp_col_id'] );
			else if ( $task['action'] == 'remove_media' )
				do_action( 'wplr_remove_media_from_collection', (int)$task['wp_id'], (int)$task['wp_col_id'] );
			else if ( $task['action'] == 'add_tag' ) {
				$pTagParent = $wplr->get_meta( 'tag_parent', (int)$task['id'] );
				do_action( 'wplr_add_tag', (int)$task['id'], $task['name'], $pTagParent );
			}
			else if ( $task['action'] == 'remove_tag' )
				do_action( 'wplr_remove_tag', (int)$task['id'] );
			else if ( $task['action'] == 'update_tag' )
				do_action( 'wplr_update_tag', (int)$task['id'], $task['name'], (int)$task['parent'] );
			else if ( $task['action'] == 'add_media_tag' )
				do_action( 'wplr_add_media_tag', (int)$task['wp_id'], (int)$task['id'] );
			else if ( $task['action'] == 'remove_media_tag' )
				do_action( 'wplr_remove_media_tag', (int)$task['wp_id'], (int)$task['id'] );

			echo json_encode( array( 'success' => true ) );
		}
		catch ( Exception $e ) {
			error_log( print_r( $e, true ) );
			echo json_encode( array( 'success' => false, 'message' => "An exception was caught and written in the PHP error logs." ) );
		}
		die();
	}

	function wplrsync_extensions_init() {
		global $wpdb, $wplr;
		$isRemoval = isset( $_POST['isRemoval'] ) && $_POST['isRemoval'] == 1;
		$tbl_m = $wpdb->prefix . 'lrsync';
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$tbl_t = $wpdb->prefix . 'lrsync_meta';

		// Collections
		$tasks = $wplr->read_collections_recursively( null, array(), $isRemoval );

		// Photos
		foreach ( $tasks as $c ) {
			if ( !$c['is_folder'] ) {
				$photos = $wpdb->get_results( $wpdb->prepare( "SELECT wp_id, wp_col_id, sort FROM $tbl_r WHERE wp_col_id = %d ORDER BY sort", $c['wp_col_id'] ), ARRAY_A );
				foreach ( $photos as $p )
					array_push( $tasks, array_merge( array( 'action' => $isRemoval ? 'remove_media' : 'add_media' ), $p ) );
			}
		};

		// Tags
		$tags = $wpdb->get_results( $wpdb->prepare( "SELECT id as id, value as name FROM $tbl_t WHERE name = %s", 'tag_name' ), ARRAY_A );
		foreach ( $tags as $tag )
			array_push( $tasks, array_merge( array( 'action' => $isRemoval ? 'remove_tag' : 'add_tag' ), $tag ) );

		// Tag's Parents
		if ( !$isRemoval ) {
			$tags = $wpdb->get_results( $wpdb->prepare( "SELECT m1.id as id, m1.value as name, m2.value as parent
				FROM $tbl_t m1
				JOIN $tbl_t m2 ON m2.id = m1.id AND m2.name = %s
				WHERE m1.name = %s", 'tag_parent', 'tag_name' ), ARRAY_A );

			foreach ( $tags as $tag )
				array_push( $tasks, array_merge( array( 'action' => 'update_tag' ), $tag ) );
		}

		// Media Tags
		$tags = $wpdb->get_results( $wpdb->prepare( "
			SELECT id as wp_id, value as id
			FROM $tbl_t
			WHERE name = %s", 'media_tag' ), ARRAY_A );
		foreach ( $tags as $tag )
			array_push( $tasks, array_merge( array( 'action' => $isRemoval ? 'remove_media_tag' : 'add_media_tag' ), $tag ) );
		echo json_encode( array( 'success' => true, 'data' => $isRemoval ? $tasks : array_reverse( $tasks ) ) );
		die();
	}

	function admin_maintenance_callback() {
		?>
			<script type="text/javascript">

				function wplr_do_next(tasks, isRemoval) {
					var tasks = tasks;
					var task = tasks.pop();
					if (!task) {
						if ( isRemoval )
							wplr_final_reset();
						else
							jQuery('#wplr-status').text("Done!");
						return;
					}
					if (task.action == 'add_collection') {
						$collectionOrFolder = task.is_folder == '1' ? 'folder' : 'collection';
						jQuery('#wplr-status').text("Create " + $collectionOrFolder + " '" + task.name + "'...");
					}
					else if (task.action == 'remove_collection') {
						$collectionOrFolder = task.is_folder == '1' ? 'folder' : 'collection';
						jQuery('#wplr-status').text("Remove " + $collectionOrFolder + " '" + task.name + "'...");
					}
					else if (task.action == 'add_media') {
						jQuery('#wplr-status').text("Add media " + task.wp_id + " to collection " + task.wp_col_id + "...");
					}
					else if (task.action == 'remove_media') {
						jQuery('#wplr-status').text("Remove media " + task.wp_id + " from collection " + task.wp_col_id + "...");
					}
					else if (task.action == 'add_tag') {
						jQuery('#wplr-status').text("Add tag " + task.name + "...");
					}
					else if (task.action == 'update_tag') {
						jQuery('#wplr-status').text("Add parent " + task.parent + " for tag " + task.name + "...");
					}
					else if (task.action == 'add_media_tag') {
						jQuery('#wplr-status').text("Add tag " + task.id + " to media " + task.wp_id + "...");
					}
					else if (task.action == 'remove_media_tag') {
						jQuery('#wplr-status').text("Remove tag " + task.id + " from media " + task.wp_id + "...");
					}
					else if (task.action == 'remove_tag') {
						jQuery('#wplr-status').text("Remove tag " + task.name + "...");
					}
					else {
						jQuery('#wplr-status').text("Running unknown tasks...");
					}
					jQuery.post(ajaxurl, {
						action: 'wplrsync_extensions_query',
						isRemoval: 0,
						isAjax: true,
						task: task
					}, function (response) {
						var r = jQuery.parseJSON(response);
						if (!r)
							alert("Error on the server. Check your PHP Error Logs.");
						else if (r.success === false)
							alert(r.message);
						else {
							wplr_do_next(tasks, isRemoval);
						}
					});
				}

				function wplr_reset() {
					jQuery('#wplr-status').text("Initiating reset...");
					jQuery.post(ajaxurl, {
						action: 'wplrsync_extensions_init',
						isRemoval: 1,
						isAjax: true
					}, function (response) {
						var r = jQuery.parseJSON(response);
						if (r.success === false)
							alert(r.message);
						else {
							wplr_do_next(r.data, true);
						}
					});
				}

				function wplr_final_reset() {
					jQuery('#wplr-status').text("Final reset...");
					jQuery.post(ajaxurl, {
						action: 'wplrsync_extensions_reset',
						isAjax: true
					}, function (response) {
						var r = jQuery.parseJSON(response);
						if (r.success === false)
							alert(r.message);
						else {
							jQuery('#wplr-status').text("Done!");
						}
					});
				}

				function wplr_create() {
					jQuery('#wplr-status').text("Initiating sync...");
					jQuery.post(ajaxurl, {
						action: 'wplrsync_extensions_init',
						isRemoval: false,
						isAjax: true
					}, function (response) {
						var r = jQuery.parseJSON(response);
						if (r.success === false)
							alert(r.message);
						else {
							wplr_do_next(r.data);
						}
					});
				}

				function wplr_clean() {
					jQuery('#wplr-status').text("Cleaning...");
					jQuery.post(ajaxurl, {
						action: 'wplrsync_clean',
						isAjax: true
					}, function (response) {
						var r = jQuery.parseJSON(response);
						if (r.success === false)
							alert(r.message);
						else {
						jQuery('#wplr-status').text("Done!");
						}
					});
				}

			</script>
		<?php
		$html = '<div class="button button-primary" onclick="wplr_create()">' .
			__( 'Resync with Extensions', 'wplr-sync' ) . '</div> ';
		$html .= '<div class="button" onclick="wplr_reset()">' .
			__( 'Reset with Extensions', 'wplr-sync' ) . '</div> ';
		$html .= '<div class="button" onclick="wplr_clean()">' .
			__( 'Clean DB', 'wplr-sync' ) . '</div> ';
		$html .= '<small style="
			color: #fffffc;
	    display: block;
	    border: 1px solid;
	    margin-top: 20px;
	    padding: 5px 10px;
	    margin-bottom: 20px;
	    background: #0094ba;
	    border-radius: 10px;
    	text-align: center;" id="wplr-status"></small>';

		$html .= __( '<span class="description"><b>Resync with Extensions</b> will request the extensions to re-create the folders/collections/keywords structure.</label><br /><br /><b>Reset with Extensions</b> will request the extensions to cancel/remove their usage of the folders/collections/keywords.', 'wplr-sync' ) . '</div> ';

		echo $html;
	}

	function admin_protocol_callback( $args ) {
		$html = '<select id="wplr_protocol" name="wplr_protocol">';
		foreach ( $args as $arg )
			$html .= '<option value="' . $arg . '"' . selected( $arg, get_option( 'wplr_protocol' ), false ) . ' > '  . $arg . '</option><br />';
		$html .= '</select><br />';
		$html .= '<span class="description">' .
			__( 'The default used to be XML/RPC but let\'s try to switch to HTTP API.', 'wplr-sync' ) .
			'</span>';
		echo $html;
	}

	function admin_takendate_callback( $args ) {
		$html = '<input type="checkbox" id="wplr_use_taken_date" name="wplr_use_taken_date" value="1" ' .
			checked( 1, get_option( 'wplr_use_taken_date' ), false ) . '/>';
		$html .= '<label for="wplr_use_taken_date"> '  . $args[0] . '</label><br>';
		$html .= '<span class="description">' .
			__( 'The date of the Media will not be the time when it was added to the Media Library but the the time when the photo was taken.', 'wplr-sync' ) . ' ';
		$html .= function_exists( 'exif_read_data' ) ?
			__( 'EXIF functions are enabled on your server.', 'wplr-sync' ) :
			__( 'EXIF functions are <b>NOT</b> enabled on your server.', 'wplr-sync' );
		$html .= '</span>';
		echo $html;
	}

	function admin_enable_keywords_callback( $args ) {
		$html = '<input type="checkbox" id="wplr_enable_keywords" name="wplr_enable_keywords" value="1" ' .
			checked( 1, get_option( 'wplr_enable_keywords' ), false ) . '/>';
		$html .= '<label for="wplr_enable_keywords"> ' . $args[0] . '</label><br>';
		$html .= '<span class="description">' .
			__( 'The Keywords taxonomy (<b>attachment_keyword</b>) will be added to your Media Library. It works the same way as the tags already available for your posts.', 'wplr-sync' ) . '</span>';
		echo $html;
	}

	function admin_sync_keywords_callback( $args ) {
		$taxonomies = get_object_taxonomies( 'attachment' );
    array_unshift( $taxonomies, "" );
		$html = '<select id="wplr_sync_keywords" name="wplr_sync_keywords">';
		foreach ( $taxonomies as $arg )
			$html .= '<option value="' . $arg . '"' .
				selected( $arg, get_option( 'wplr_sync_keywords' ), false ) . ' > '  .
				( empty( $arg ) ? 'none' : $arg ) . '</option><br />';
		$html .= '</select><br />';
		$html .= '<span class="description">' .
			__( 'Synchronize your LR keywords with a taxonomy available in your Media Library (through your theme or another plugin). If the above option is checked, the Keywords (<b>attachment_keyword</b>) will be available.', 'wplr-sync' ) . '</span>';
		echo $html;
	}

	function admin_library_show_hierarchy_callback( $args ) {
		$html = '<input type="checkbox" id="wplr_library_show_hierarchy" name="wplr_library_show_hierarchy" value="1" ' .
			checked( 1, get_option( 'wplr_library_show_hierarchy' ), false ) . '/>';
		$html .= '<label for="wplr_library_show_hierarchy"> '  . $args[0] . '</label><br>';
		$html .= '<span class="description">' .
			__( 'Add a filter in the Media Library to allows selection of a particular collection (also display the hierarchy of folders).', 'wplr-sync' ) . '</span>';
		echo $html;
	}

	function admin_filename_accents_callback( $args ) {
		$html = '<input type="checkbox" id="wplr_filename_accents" name="wplr_filename_accents" value="1" ' .
			checked( 1, get_option( 'wplr_filename_accents' ), false ) . '/>';
		$html .= '<label for="wplr_filename_accents"> '  . $args[0] . '</label><br>';
		$html .= '<span class="description">' .
			__( 'By default, the accents are removed from the filenames, spaces are turned into hyphens and all is converted to small caps (to avoid issues on many installs). Enable this to avoid your filenames to be changed from what your picked in LR.', 'wplr-sync' ) . '</span>';
		echo $html;
	}

	function admin_debugging_callback( $args ) {
		$html = '<input type="checkbox" id="wplr_debugging_enabled" name="wplr_debugging_enabled" value="1" ' .
			checked( 1, get_option( 'wplr_debugging_enabled' ), false ) . '/>';
		$html .= '<label for="wplr_debugging_enabled"> '  . $args[0] . '</label><br>';
		$html .= '<span class="description">' .
			__( 'Add a Debugging Tools menu in WP/LR Sync. It will also display the WP/LR Sync extensions and related menus that might have been hidden by your theme. For advanced users only.', 'wplr-sync' ) . '</span>';
		echo $html;
	}

	function admin_debuglogs_callback( $args ) {
		$clearlogs = isset ( $_GET[ 'clearlogs' ] ) ? $_GET[ 'clearlogs' ] : 0;
		if ( $clearlogs ) {
			if ( file_exists( plugin_dir_path( __FILE__ ) . '/wplr-sync.log' ) ) {
				unlink( plugin_dir_path( __FILE__ ) . '/wplr-sync.log' );
			}
		}
		$html = '<input type="checkbox" id="wplr_debuglogs" name="wplr_debuglogs" value="1" ' .
			checked( 1, get_option( 'wplr_debuglogs' ), false ) . '/>';
		$html .= '<label for="wplr_debuglogs"> '  . $args[0] . '</label><br>';
		$html .= '<span class="description">' . __( 'Create an internal log file. For advanced users only.', 'wplr-sync' );
		if ( file_exists( plugin_dir_path( __FILE__ ) . '/wplr-sync.log' ) ) {
			$html .= sprintf( __( '<br />The <a target="_blank" href="%s/wplr-sync.log">log file</a> is available. You can also <a href="?page=wplr-main-menu&clearlogs=true">clear</a> it.', 'wplr-sync' ), plugin_dir_url( __FILE__ ) );
		}
		$html .= '</span>';
		echo $html;
	}

	function admin_public_api_callback( $args ) {
		$html = '<input type="checkbox" id="wplr_public_api" name="wplr_public_api" value="1" ' .
			checked( 1, get_option( 'wplr_public_api', true ), false ) . '/>';
		$html .= '<label for="wplr_public_api"> '  . $args[0] . '</label><br>';
		$html .= '<span class="description">' .
			__( 'Add functions in the WordPress API that can be reach by external apps (mobile apps for instance). A QR Code will be displayed in the admin and need to be used for authentification.', 'wplr-sync' );
		if ( get_option( 'wplr_public_api', true ) ) {
			$wplr_token = get_transient( 'wplr_auth_token' );
			$html .= ' Quick test: <a target="_blank" href="/wp-json/wplr/v1/hierarchy?wplr_auth_token='
				. $wplr_token . '">Hierarchy</a>.';
		}
		$html .= '</span>';
		echo $html;
	}

	function display_image_box( $wpid, $image = null, $showignore = false ) {
		$metadata = wp_get_attachment_image_src( $wpid, "full", false );
		$parent = wp_get_post_parent_id( $wpid );
		$title = get_the_title( $wpid  );
		$parent_title = $parent ? get_the_title( $parent ) : "";
		$filename = $metadata ? $metadata[0] : "";
		echo "<div id='wplr-image-box-$wpid' class='wplr-image-box'>";
		echo "<div class='wplr-title-wpid'>";
		echo "Media #<a target='_blank' title='$title' href='post.php?post=$wpid&action=edit'>$wpid</a>";
		if ( $parent ) {
			echo "<span style='float: right;'>Post #<a title='$parent_title' target='_blank' href='post.php?post=$parent&action=edit'>$parent</a></span><br />";
		}
		echo "</div>";
		echo "<div class='wplr-image'>";
		echo "<a target='_blank' title='$title' href='$filename'>" . wp_get_attachment_image( $wpid, array( 190, 190 ), false, array( 'alt' => $title ) ) . "</a>";
		echo "</div>";
		echo "<div style='clear: both;'></div>";
		echo "<span class='wplr-actions'>";
		if ( $image ) {
			echo "<a style='background: #B34A4A;' href='admin.php?page=wplr-tools-menu&show=duplicates&action=unlink&wpid=$wpid&lrid=$image->lr_id'>Unlink</a>";
			echo "<a style='' href='admin.php?page=wplr-tools-menu&show=duplicates&action=delete&wpid=$wpid&lrid=$image->lr_id'>Delete</a>";
		}
		else {
			echo "<div>
				LR ID:
				<input type='text' class='wplr-sync-lrid-input wplrsync-link-" . $wpid . "'></input>
				<div class='wplr-button wplr-button-link' onclick='wplrsync_link($wpid)'>Link</div>";
			if ( $showignore )
				echo "<div class='wplr-button wplr-button-link' onclick='wplrsync_link($wpid, true)'>Ignore</div>";
			echo "</div>";
		}
		echo "</span>";
		echo "</div>";
	}

	function admin_collections_keywords() {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$this->admin_css();
		?>
		<div class='wrap'>

			<div class="right-25">
				<div style="padding: 2px 20px; box-shadow: 0px 2px 5px #d4d4d4; background: white; top: 55px; position: fixed; margin-right: 20px;">
					<h3>Galleries (Shortcodes)</h3><br />
					<p><?php _e( 'You can use new attributes with the standard WP Gallery shortcode. <i>Hover over the collections, folders and tags.</i>', 'wplr-sync' ); ?></p><br />
					<div class="shortcode-preview" id="shortcode-preview-code" style="display: none;">
					</div>
					<p><?php _e( 'To make your galleries better, the <b><a target="_blank" href="https://wordpress.org/plugins/meow-gallery/">Meow Gallery</a></b> plugin is recommended. It works naturally with the standard WP Gallery, adds features to it and supports many kinds of layouts :)', 'wplr-sync' ); ?></p>
				</div>
			</div>
			<div class="left-75">
				<?php echo $this->display_title( __( 'Collections & Keywords', 'wplr-sync' ) . " | WP/LR Sync" );  ?>
				<br />
				<p style="margin-bottom: 18px;">
					<?php _e( 'This is the hierarchy of folders, collections and tags stored by WP/LR Sync internally. If you wish this hierarchy to be synchronized with your theme directly, please learn about the <a target="_blank" href="https://meowapps.com/wplr-sync/wplr-extensions/">WP/LR Sync Extensions</a>. You can also try the <a target="_blank" href="https://wordpress.org/plugins/meow-gallery/">Meow Gallery</a></b> plugin that makes use of all this natively.', 'wplr-sync' ); ?>
				</p>
				<p class="buttons">
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="hierarchy">
						<input type="submit" name="submit" id="submit" class="button button-primary"
							value="<?php _e( 'Folders & Collections', 'wplr-sync' ) ?>">
					</form>
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="keywords">
						<input type="submit" name="submit" id="submit" class="button button-primary"
							value="<?php _e( 'Keywords (Tags)', 'wplr-sync' ) ?>">
					</form>
				</p><br /	>
				<?php $this->display_hierarchy( isset( $_POST['action'] ) ? $_POST['action'] : null ); ?>
			</div>
		</div>
		<?php
	}

	function admin_tools() {
		$images = array();
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$show = null;
		if ( isset( $_GET['action'] ) ) {
			$wpid = isset( $_GET['wpid'] ) ? $_GET['wpid'] : null;
			$lrid = isset( $_GET['lrid'] ) ? $_GET['lrid'] : null;
			if ( $_GET['action'] == "link" ) {
				$this->wplr->link_media( $lrid, $wpid );
			}
			else if ( $_GET['action'] == "unlink" ) {
				$this->wplr->unlink_media( $lrid, $wpid );
			}
			else if ( $_GET['action'] == "delete" ) {
				$this->wplr->unlink_media( $lrid, $wpid );
				wp_delete_attachment( $wpid );
			}
		}

		if ( isset( $_GET['show'] ) && $_GET['show'] == "duplicates" ) {
			$show = "duplicates";
			$images = $this->wplr->list_duplicates();
		}
		else if ( isset( $_GET['show'] ) && $_GET['show'] == "ignored" ) {
			$show = "ignored";
			$images = $this->wplr->list_ignored();
		}
		else {
			$show = "unlinked";
			$images = $this->wplr->list_unlinks( true );
		}

		?>
			<style>

				.wplr-title-lrid {
					background: #515151;
					color: white;
					padding: 0px 2px;
					margin-left: 5px;
				}

				.wplr-duplicates-box {
					padding: 5px;
					border: 2px solid  #515151;
					border-radius: 5px;
					margin-bottom: 10px;
				}

				.wplr-image-box {
					width: 190px;
					margin-right: 10px;
					float: left;
					font-size: 10px;
					background: white;
					padding: 2px 5px;
					margin-bottom: 10px;
					box-shadow: 2px 2px 2px #D7D7D7;
				}

				.wplr-image-box .wplr-image {
					width: 190px;
					height: 128px;
					overflow: hidden;
				}

				.wplr-title-wpid {
					font-size: 11px;
					line-height: 20px;
				}

				.wplr-title-filename {
					line-height: 8px;
				}

				.wplr-title-wpid a {
					text-decoration: none;
				}

				.wplr-actions a {
					margin-right: 5px;
					padding: 1px 3px;
					background: rgb(92, 92, 92);
					color: white;
				}

				.subsubsub #icl_subsubsub, .subsubsub br {
					display: none;
				}

			</style>

			<div class='wrap'>

			<?php echo $this->display_title( __( 'Media Tools', 'wplr-sync' ) . " | WP/LR Sync" );  ?>

			<p><?php _e( "This screen will help you to link your unlinked photos between WordPress and Lightroom. <b>Don't forget that if you link a photo from here you also need to also add it manually to the WP/LR Service in Lightroom.</b>
		Last but not least, be careful with the in 'Duplicated Links', they are <b>immediate</b> and the 'Delete' is <b>unrecoverable</b>.", 'wplr-sync' ); ?></p>
			<ul class="subsubsub">
				<li><a class="<?php echo $show == "unlinked" ? "current" : "" ?>" href="admin.php?page=wplr-tools-menu&show=unlinked"><?php _e( "Unlinked Photos", 'wplr-sync' ); ?></a> |</li>
				<li><a class="<?php echo $show == "duplicates" ? "current" : "" ?>" href="admin.php?page=wplr-tools-menu&show=duplicates"><?php _e( "Duplicated Links", 'wplr-sync' ); ?></a> | </li>
				<li><a class="<?php echo $show == "ignored" ? "current" : "" ?>" href="admin.php?page=wplr-tools-menu&show=ignored"><?php _e( "Ignored Photos", 'wplr-sync' ); ?></a></li>
			</ul>
			<div style='clear: both; margin-bottom: 18px; margin-top: -5px;'></div>

		<?php

		if ( count( $images ) == 0 && $show == "unlinked" )
			echo "<div class='updated'><p>" . __( 'There are no unlinked photos.', 'wplr-sync' ) . "</p></div>";
		else if ( count( $images ) == 0 && $show == "duplicates" )
			echo "<div class='updated'><p>" . __( 'There are no duplicated links.', 'wplr-sync' ) . "</p></div>";
		else if ( count( $images ) == 0 && $show == "ignored" )
			echo "<div class='updated'><p>" . __( 'There are no ignored images.', 'wplr-sync' ) . "</p></div>";
		else {
			foreach ( $images as $image ) {
				if ( $show == "duplicates" ) {
					echo "<b class='wplr-title-lrid'>LR ID " . $image->lr_id . "</b> Filename: " . ( $image->lr_file ? $image->lr_file : "Unknown" );
					echo "<div class='wplr-duplicates-box'>";
					$wpids = explode( ',', $image->wpids );
					foreach ( $wpids as $wpid ) {
						$this->display_image_box( $wpid, $image, true );
					}
					echo "<div style='clear: both;'></div>";
					echo "</div>";
				}
				else if ( $show == "unlinked" ) {
					$this->display_image_box( $image->ID, null, true );
				}
				else if ( $show == "ignored" ) {
					$this->display_image_box( $image->ID, null, false );
				}
			}
		}

		echo "</div>";
	}

	function display_hierarchy( $action ) {
		$mode = isset( $_POST['action'] ) && $action == "keywords" ? 'keywords' : 'hierarchy';

		if ( isset( $_POST['delete_collection'] ) ) {
			global $wplr;
			$wplr->delete_collection( $_POST['delete_collection'] );
		}
		if ( isset( $_POST['delete_keyword'] ) ) {
			global $wplr;
			$wplr->delete_keyword( $_POST['delete_keyword'] );
		}

		echo "<br /><h3>" . ( $mode == 'keywords' ?
			__( "Keywords (Tags)", 'wplr-sync' ) :
			__( "Folders & Collections", 'wplr-sync' ) ) . "</h3><br />";
		$hierarchy = $mode == 'keywords' ? $this->wplr->get_keywords_hierarchy() : $this->wplr->get_hierarchy();

		function display_internal_hierarchy( $hierarchy, $level = 0, $mode ) {
			global $wpdb;
			$tbl_m = $wpdb->prefix . 'lrsync';
			$tbl_r = $wpdb->prefix . 'lrsync_relations';
			$tbl_c = $wpdb->prefix . 'lrsync_collections';
			$page = empty( $_GET['page'] ) ? 'wplr-collections-tags-menu' : $_GET['page'];
			echo '<div class="hierarchy-level-' . $level . '">';
			if ( $level > 0 )
				echo '<div style="margin-left: 10px; border-left: 1px dotted gray; padding-left: 10px; margin-bottom: 5px;">';
			foreach ( $hierarchy as $c ) {

				if ( $mode == 'keywords' ) {
					echo '<div class="wplr-elem wplr-keyword" wplr-elem-type="keyword" wplr-elem-id="' .
						$c['id'] . '"	wplr-elem-name="' . $c['name'] . '"><span class="dashicons dashicons-carrot"></span> <b>' .
						$c['name'] . '</b> <small>(ID: ' . $c['id'] . ', Photos: ' . $c['count'] . ')</small>';
					echo '<form action="" method="post" style="display: inline;">
						<input type="hidden" name="page" value="' . $page . '">
						<input type="hidden" name="action" value="' . $mode . '">
						<small class="wplr_delete" style="display: none;">
						<button type="submit" name="delete_keyword" value="' . $c['id'] . '" class="btn-link">' .
						__( 'Delete', 'wplr-sync' ) . '</button>
						</small></form>';
					echo '</div>';
				}
				else {
					$type = $c['type'] == 'folder' ? '<span class="dashicons dashicons-category"></span>' : '<span class="dashicons dashicons-admin-media"></span>';
					echo '<div class="wplr-elem wplr-' . $c['type']
							. '" wplr-elem-type="' . $c['type'] . '" wplr-elem-id="' . $c['id'] . '" wplr-elem-name="' . $c['name'] . '">'
						. $type . ' ' . $c['name'] . ' <small>(ID: ' . $c['id'] . ')</small>';

					echo '<form action="" method="post" style="display: inline;">
						<input type="hidden" name="page" value="' . $page. '">
						<small class="wplr_delete" style="display: none;">
						<button type="submit" name="delete_collection" value="' . $c['id'] . '" class="btn-link">' .
						__( 'Delete', 'wplr-sync' ) . '</button>
						</small></form>';
					echo '</div>';
				}

				if ( ( $mode == 'hierarchy' && $c['type'] == 'folder' ) ||
					( $mode == 'keywords' && count( $c['children'] ) > 0 ) ) {
					display_internal_hierarchy( $c['children'], $level + 1, $mode );
				}
				else {
					if ( $mode == 'hierarchy' ) {
						$photos = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $tbl_r WHERE wp_col_id = %d ORDER BY sort LIMIT 25", $c['id'] ), OBJECT );
						if ( count( $photos ) > 0 ) {
							echo '<div style="margin-left: 15px; padding: 2px; margin-top: 2px; border: 1px solid gray;">';
							foreach ( $photos as $photo ) {
								$src = wp_get_attachment_image_src( $photo->wp_id, 'thumbnail' );
								echo '<div><a href="media.php?attachment_id=' . $photo->wp_id . '&action=edit"><img style="float: left; margin-right: 2px;" width="32" height="32" src="' . $src[0] . '"></a></div>';
							}
							echo '<div style="clear: both;"></div></div>';
						}
					}
				}
			}
			if ( $level > 0 )
				echo '</div>';
			echo '</div>';
		}
		display_internal_hierarchy( $hierarchy, 0, $mode );
		?>
		<script>
		function wplr_toggle_delete() {
			jQuery('.wplr_delete').toggle();
		}
		function wplr_toggle_delete() {
			jQuery('.wplr_delete').toggle();
		}
		jQuery('.wplr-elem').hover(function () {
			var id = jQuery(this).attr('wplr-elem-id');
			var ids = [];
			var name = jQuery(this).attr('wplr-elem-name');
			var type = jQuery(this).attr('wplr-elem-type');
			var content = "";

			if (type == 'folder') {
				jQuery(this).next('div').find('.wplr-collection').each(function(i, v) {
					ids.push(jQuery(v).attr('wplr-elem-id'));
				});
				id = ids.join(',');
			}
			if (type == 'keyword') {
				ids.push(id);
				jQuery(this).next('div').find('.wplr-keyword').each(function(i, v) {
					ids.push(jQuery(v).attr('wplr-elem-id'));
				});
				id = ids.join(',');
			}

			if (type == 'collection' || type == 'folder') {
				content += 'Here are my photos about <i>' + name + ':</i><br />';
				content += '[gallery wplr-collection' + (ids.length > 1 ? 's' : '') + '="' + id + '"]';
			}
			else if (type == 'keyword') {
				content += 'Here are my photos about <i>' + name + ':</i><br />';
				content += '[gallery wplr-keyword' + (ids.length > 1 ? 's' : '') + '="' + id + '"]';

				// content += '<i>Keywords are not supported yet, if you are interested in this feature please come on the forum of the Meow Gallery and ask for it :) Here: <a href="https://wordpress.org/support/plugin/meow-gallery">Meow Gallery Support Forum</a><i>';
			}
			jQuery('#shortcode-preview-code').html(content);
			jQuery('#shortcode-preview-code').show();
			console.log(jQuery(this));
		});
		</script>
		<p>
			<small><input type="checkbox" name="unlocked" onclick="wplr_toggle_delete()">
				<?php _e( 'Unlock <b>delete</b>', 'wplr-sync' ) ?>
			</small>
		</p>
		<?php
	}

	function admin_css() {
		?>
		<style>

			.left {
				width: 450px;
			}

			.right {
				position: absolute;
				left: 460px;
				right: 10px;
			}

			.left-75 {
				width: 60%;
			}

			.right-25 {
				position: absolute;
				width: 34%;
				right: 20px;
			}

			.wplr-collection, .wplr-folder, .wplr-keyword {
				cursor: pointer;
			}

			.shortcode-preview {
				margin-bottom: 20px;
		    background: #0085ba;
		    padding: 10px;
		    font-size: 12px;
		    word-break: break-all;
		    color: white;
		    margin-top: -10px;
			}

			h3 {
				margin-bottom: 5px;
			}

			p {
				margin-top: 0px;
				margin-bottom: 10px;
			}

			.wplrsync-form {
				border: 1px solid lightgrey;
				padding: 5px;
				background: white;
				border-radius: 5px;
				width: 420px;
			}

			th {
				text-align: left;
			}

			#wpfooter {
				display: none;
			}

		</style>
		<?php
	}

	function admin_debug() {
		if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		// HEADER
		$this->admin_css();
		?>
		<div class="wrap" >

			<?php echo $this->display_title( __( 'Debugging Tools', 'wplr-sync' ) . " | WP/LR Sync" ); ?>
			<p>Those tools should be used for debugging purposed. Be careful when you are using them, especially the colored buttons (usually blue).</p>
		<?php

		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";

		if ( !isset( $_POST['action'] ) && isset( $_GET['action'] ) ) {
			$_POST['action'] = $_GET['action'];
		}

		if ( isset( $_POST['action'] ) ) {

			$action = $_POST['action'];
			if ( $action == "reset" ) {
				$this->wplr->reset_db();
				echo "<div class='updated'><p>" .
					__( 'The database was reset.', 'wplr-sync' ) .
					"</p></div>";
			}
			else if ( $action == "duplicates" ) {
				$duplicates = $this->wplr->list_duplicates();
			}
			else if ( $action == "scan" ) {
				$unlinks = $this->wplr->list_unlinks();
			}
			else if ( $action == "list" ) {
				$list = $this->wplr->list_sync_media();
			}
			else if ( $action == "link" ) {
				if ( $this->wplr->link_media( $_POST['lr_id'], $_POST['wp_id'] ) ) {
					echo "<div class='updated'><p>" .
						__( 'Link successful.', 'wplr-sync' ) .
						"</p></div>";
				}
				else {
					echo "<div class='error'><p>" . $this->wplr->get_error()->message . "</p></div>";
				}
			}
			else if ( $action == "linkinfo_upload" ) {
				if ( isset( $_FILES['file'], $_FILES['file']['tmp_name'] ) && !empty( $_FILES['file']['tmp_name'] ) ) {
					$tmp_path = $_FILES['file']['tmp_name'];
					$linkinfo = $this->wplr->linkinfo_upload( $tmp_path, null );
				}
				else {
					$linkinfo = $this->wplr->linkinfo_media( $_POST['wp_id'] );
				}
				if ( $linkinfo )
					echo "<div class='updated'><p>" .
						__( 'Link info retrieved.', 'wplr-sync' )
						. "</p></div>";
				else
					echo "<div class='error'><p>" . $this->wplr->get_error()->message . "</p></div>";
			}
			else if ( $action == "sync" ) {
				$tmp_path = $_FILES['file']['tmp_name'];
				$lrinfo = new Meow_WPLR_LRInfo();
				$lrinfo->lr_id = $_POST['lr_id'] == "" ? -1 : $_POST['lr_id'];
				$lrinfo->lr_file = $_FILES['file']['name'];
				$lrinfo->lr_title = isset( $_POST['title'] ) ? $_POST['title'] : "";
				$lrinfo->lr_caption = $_POST['caption'];
				$lrinfo->lr_desc = isset( $_POST['desc'] ) ? $_POST['desc'] : "";
				$lrinfo->lr_alt_text = isset( $_POST['altText'] ) ? $_POST['altText'] : "";
				$lrinfo->sync_title = isset( $_POST['syncTitle'] ) && $_POST['syncTitle'] == 'on';
				$lrinfo->sync_caption = isset( $_POST['syncCaption'] ) && $_POST['syncCaption'] == 'on';
				$lrinfo->sync_desc = isset( $_POST['syncDesc'] ) && $_POST['syncDesc'] == 'on';
				$lrinfo->sync_alt_text = isset( $_POST['syncAltText'] ) && $_POST['syncAltText'] == 'on';
				$lrinfo->type = $_FILES['file']['type'];
				if ( $this->wplr->sync_media( $lrinfo, $tmp_path ) ) {
					echo "<div class='updated'><p>";
					echo sprintf( __( 'Lr ID %d was synchronized with the attachment.', 'wplr-sync' ), $_POST['lr_id'] );
					echo "</p></div>";
				}
				else
					echo "<div class='error'><p>" . $this->wplr->get_error()->message . "</p></div>";

			}
			else if ( $action == "unlink" ) {
				if ( $this->wplr->unlink_media( $_POST['lr_id'], $_POST['wp_id'] ) ) {
					echo "<div class='updated'><p>";
					echo sprintf( __( 'Media ID %d was unlinked.', 'wplr-sync' ), $_POST['lr_id'] );
					echo "</p></div>";
				}
				else
					echo "<div class='error'><p>" . $this->wplr->get_error()->message . "</p></div>";
			}
			else if ( $action == "remove" ) {
				if ( $this->wplr->delete_media( $_POST['lr_id'] ) ) {
					echo "<div class='updated'><p>";
					echo sprintf( __( 'Media ID %d was removed.', 'wplr-sync' ), $_POST['lr_id'] );
					echo "</p></div>";
				}
				else
					echo "<div class='error'><p>" . $this->wplr->get_error()->message . "</p></div>";
			}
			else if ( $action != 'api' && $action != 'keywords' && $action != 'hierarchy'  ) {
				echo "<div class='error'<p>Unknown action.</p></div>";
			}

		}

		// CONTENT & FORMS
		?>
			<div class="right">

				<h3><?php _e( 'Display', 'wplr-sync' ); ?></h3>
				<p class="buttons">
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="hierarchy">
						<input type="submit" name="submit" id="submit" class="button" value="Show Hierarchy">
					</form>
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="keywords">
						<input type="submit" name="submit" id="submit" class="button" value="Show Keywords">
					</form>
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="list">
						<input type="submit" name="submit" id="submit" class="button" value="List Linked">
					</form>
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="scan">
						<input type="submit" name="submit" id="submit" class="button" value="List Unlinked">
					</form>
					<form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="duplicates">
						<input type="submit" name="submit" id="submit" class="button" value="List Duplicates">
					</form>
					<!-- <form style="float: left; margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="api">
						<input type="submit" name="submit" id="submit" class="button" value="API Ping">
					</form> -->
				</p>
				<br />

				<?php
					if ( isset( $_POST['action'] ) && $action == "scan" ) {
						echo '<br /><h3>' . __( 'Unlinked Media', 'wplr-sync' ) . '</h3>';
						echo '<pre>';
						print_r( $unlinks );
						echo '</pre>';
					}
					else if ( isset( $_POST['action'] ) && $action == "list" ) {
						echo '<br /><h3>' . __( 'Linked Media', 'wplr-sync' ) . '</h3>';
						echo '<pre>';
						print_r( $list );
						echo '</pre>';
					}
					else if ( isset( $_POST['action'] ) && $action == "linkinfo_upload" ) {
						echo '<pre>';
						print_r( $linkinfo );
						echo '</pre>';
					}
					else if ( isset( $_POST['action'] ) && $action == "duplicates" ) {
						echo '<br /><h3>' . __( 'Duplicates', 'wplr-sync' ) . '</h3>';
						echo '<pre>';
						print_r( $duplicates );
						echo '</pre>';
					}
					else if ( isset( $_POST['action'] ) && $action == "api" ) {
						$boundary = wp_generate_password(24);
						$response = wp_remote_post( admin_url( 'admin-ajax.php' ),
							array(
								'headers' => array( 'content-type' => 'multipart/form-data; boundary=' . $boundary ),
								'body' => array( 'action' => 'lrsync_api', 'isAjax' => 1, 'data' => '' )
						  )
						);
						echo '<br /><h3>' . __( 'API Response', 'wplr-sync' ) . '</h3>';
						echo '<pre>';
						print_r( $response );
						echo '</pre>';
					}
					else {
						$this->display_hierarchy( isset( $_POST['action'] ) ? $action : null );
					}
				?>
			</div>

			<div class="left">
				<?php if ( isset( $link_files_count ) && ( $unlink_files_count ) ): ?>
				<div style="font-weight: bold;">
					<?php
						sprintf( __( '%d linked files out of %d media files.', 'wplr-sync' ),
							$link_files_count, $link_files_count + $unlink_files_count );
					?>
				</div>
				<?php endif; ?>

				<h3><?php _e( 'Link Info', 'wplr-sync' ) ?></h3>
				<p><?php _e( 'Link info for either a WP ID <b>or</b> the Uploaded File.', 'wplr-sync' ) ?></p>
				<form class="wplrsync-form" method="post" action="" enctype="multipart/form-data">
					<input type="hidden" name="action" value="linkinfo_upload">
					<table>
						<tr>
							<th scope="row"><label for="wp_id">WP ID</label></th>
							<td><input name="wp_id" type="text" id="wp_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="file">File</label></th>
							<td><input name="file" type="file" id="file"></td>
						</tr>
						<tr>
							<th scope="row"></th>
							<td><input type="submit" name="submit" id="submit" class="button" value="Check"></td>
						</tr>
					</table>
				</form>

				<h3><?php _e( 'Link', 'wplr-sync' ) ?></h3>
				<p><?php _e( 'Will link the WP Media ID to the Lr ID.', 'wplr-sync' ) ?></p>
				<form class="wplrsync-form" method="post" action="" enctype="multipart/form-data">
					<input type="hidden" name="action" value="link">
					<table>
						<tr>
							<th scope="row"><label for="lr_id">Lr ID</label></th>
							<td><input name="lr_id" type="text" id="lr_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="wp_id">Media ID</label></th>
							<td><input name="wp_id" type="text" id="wp_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"></th>
							<td><input type="submit" name="submit" id="submit" class="button button-primary" value="Link Media ID + Lr ID"></td>
						</tr>
					</table>

				</form>

				<h3><?php _e( 'Unlink', 'wplr-sync' ) ?></h3>
				<p><?php _e( 'Will unlink the media.', 'wplr-sync' ) ?></p>
				<form class="wplrsync-form" method="post" action="">
					<input type="hidden" name="action" value="unlink">
					<table>
						<tr>
							<th scope="row"><label for="lr_id">Lr ID</label></th>
							<td><input name="lr_id" type="text" id="lr_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="wp_id">Media ID</label></th>
							<td><input name="wp_id" type="text" id="wp_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"></th>
							<td><input type="submit" name="submit" id="submit" class="button button-primary" value="Unlink"></td>
						</tr>
					</table>
				</form>

				<h3><?php _e( 'Sync', 'wplr-sync' ) ?></h3>
				<p><?php _e( 'Will create the entry if doesn\'t exist, will update it if exists.', 'wplr-sync' ) ?></p>
				<form class="wplrsync-form" method="post" action="" enctype="multipart/form-data">
					<input type="hidden" name="action" value="sync">
					<table>
						<tr>
							<th scope="row"><label for="lr_id">Lr ID</label></th>
							<td><input name="lr_id" type="text" id="lr_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="title">Title</label></th>
							<td><input name="title" type="text" id="title" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="title">Caption</label></th>
							<td><input name="caption" type="text" id="caption" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="title">Desc</label></th>
							<td><input name="desc" type="text" id="desc" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="title">Alt</label></th>
							<td><input name="altText" type="text" id="altText" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"><label for="file">File</label></th>
							<td><input name="file" type="file" id="file"></td>
						</tr>
						<tr>
							<th scope="row"><label for="sync">Sync</label></th>
							<td>
								<label><input type="checkbox" id="syncTitle" name="syncTitle">Title</label>
								<label><input type="checkbox" id="syncCaption" name="syncCaption">Caption</label>
								<label><input type="checkbox" id="syncDesc" name="syncDesc">Desc</label>
								<label><input type="checkbox" id="syncAltText" name="syncAltText">Alt</label>
							</td>
						</tr>
						<tr>
							<th scope="row"></th>
							<td><input type="submit" name="submit" id="submit" class="button button-primary" value="Sync Media"></td>
						</tr>
					</table>
				</form>

				<h3><?php _e( 'Remove', 'wplr-sync' ) ?></h3>
				<p><?php _e( 'Will remove the media.', 'wplr-sync' ) ?></p>
				<form class="wplrsync-form" method="post" action="">
					<input type="hidden" name="action" value="remove">
					<table>
						<tr>
							<th scope="row"><label for="lr_id">Lr ID</label></th>
							<td><input name="lr_id" type="text" id="lr_id" value="" class="regular-text code"></td>
						</tr>
						<tr>
							<th scope="row"></th>
							<td><input type="submit" name="submit" id="submit" class="button button-primary" value="Remove Media"></td>
						</tr>
					</table>
				</form>

				<h3><?php _e( 'Actions', 'wplr-sync' ) ?></h3>
				<p class="buttons">
					<?php _e( 'Be careful. Those buttons are dangerous.', 'wplr-sync' ) ?>
					<form style="margin-right: 5px;" method="post" action="">
						<input type="hidden" name="action" value="reset">
						<input type="submit" name="submit" id="submit" class="button button-primary"
							value="<?php _e( 'Reset WP/LR Sync DB', 'wplr-sync' ) ?>">
					</form>
				</p>

			</div>
		</div>
	</div>
		<?php
	}
}

?>
