<?php

class Meow_WPLR_Sync_API {

	public $methods = array( 'ping', 'presync', 'sync', 'sync_collection',
		'list', 'delete', 'delete_collection', 'list_unlinks', 'link', 'unlink',
		'linkinfo', 'order_collection', 'linkinfo_upload', 'userinfo', 'list_wpids' );

	private $running = true;
	private $result = null;
	private $ob = false;
	public static $init = false;

	public function __construct() {
		if ( !self::$init ) {
			if ( strpos( $_SERVER['REQUEST_URI'], 'wp-json/wplr/v1/' ) !== false ) {
				global $wplr;
				$ob_handlers = ob_list_handlers();
				if ( count( $ob_handlers ) > 0 ) {
					$wplr->log( "[OB-OFF] " . $_SERVER['REQUEST_URI'] );
				}
				else {
					$wplr->log( "[OB] START " . $_SERVER['REQUEST_URI'] );
					$start = ob_start( array( $this, 'buffer_end' ) );
					if ( $start ) {
						$this->ob = true;
					}
					else {
						$wplr->log( "[OB-FAIL] " . $_SERVER['REQUEST_URI'] );
					}
				}
			}
		}
		self::$init = true;
	}

	function buffer_end( $buffer ) {
		return "";
	}

	function preprocess( $args = null ) {
		$this->running = true;
		// if ( version_compare( PHP_VERSION, '5.2.0' ) >= 0 ) {
		// 	register_shutdown_function( array( $this, "fatal_handler" ) );
		// }
		return $args;
	}

	function postprocess() {
		if ( $this->ob ) {
			global $wplr;
			$content = ob_get_contents();
			if ( !empty( $content ) ) {
				error_log( "There was unwanted activity during the WP/LR Sync process (probably generated by another plugin). Request: " . $_SERVER['REQUEST_URI'] . ". The output which was caught will follow." );
				$wplr->log( "There was unwanted activity during the WP/LR Sync process (probably generated by another plugin). Request: " . $_SERVER['REQUEST_URI'] . ". The output which was caught will follow.\r\n" . $content, true );
			}
			if ( !ob_end_clean() )
				$wplr->log( "[OB] CLEAN FAILED" );
			$wplr->log( "[OB] END (OB level is now " . ob_get_level() . ")" );
			if ( ob_get_level() > 0 ) {
				while ( @ob_end_clean() ) {}
				$wplr->log( "[OB] ALL END" );
			}
		}
		$this->running = false;
	}

	// function fatal_handler() {
	// 	if ( !$this->running )
	// 		return false;
	//   $errfile = "unknown";
	//   $errstr  = "unknown";
	//   $errno   = E_CORE_ERROR;
	//   $errline = 0;
	//   $error = error_get_last();
	//
	//   if ( $error !== NULL) {
	//     $errno   = $error["type"];
	//     $errfile = $error["file"];
	//     $errline = $error["line"];
	//     $errstr  = $error["message"];
	//   }
	//
	// 	$result = json_encode( array(
	// 		'success' => false,
	// 		'message' => "A PHP error occured and was caught by WP/LR Sync. Check your WPLRSync.log in your Documents directory for more information.",
	// 		'error' => array(
	// 			'type' => $errno,
	// 			'file' => $errfile,
	// 			'line' => $errline,
	// 			'message' => $errstr
	// 			)
	// 		)
	// 	);
	// 	die;
	// }

	// Authenticate and share the useful arguments.
	function init_with( &$args ) {
		$this->error = new IXR_Error( 403, __( 'Either XML/RPC or REST should be used.' ) );
		return false;
	}

	// Ping for the client. Should probably send back the version of the plugin.
	function ping( $args ) {
		global $wplr;
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->result = "Pong! Authentification is missing. Version: {$wplr->version}.";
		}
		else {
			$this->postprocess();
			return $this->result =  "Pong to you, " . $user->data->display_name . ". Version: {$wplr->version}.";
		}
	}

	// Sync file
	function presync( $args ) {
		global $wplr;
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return false;
		}
		if ( defined ('HHVM_VERSION' ) ) {
			$max_execution_time = ini_get( 'max_execution_time' ) ? (int)ini_get( 'max_execution_time' ) : 30;
			$post_max_size = ini_get( 'post_max_size' ) ? (int)$wplr->parse_ini_size( ini_get( 'post_max_size' ) ) : (int)ini_get( 'hhvm.server.max_post_size' );
			$upload_max_filesize = ini_get( 'upload_max_filesize' ) ? (int)$wplr->parse_ini_size( ini_get( 'upload_max_filesize' ) ) : (int)ini_get( 'hhvm.server.upload.upload_max_file_size' );
		}
		else {
			$max_execution_time = (int)ini_get( 'max_execution_time' );
			$post_max_size = (int)$wplr->parse_ini_size( ini_get( 'post_max_size' ) );
			$upload_max_filesize = (int)$wplr->parse_ini_size( ini_get( 'upload_max_filesize' ) );
		}
		$max = min( $post_max_size, $upload_max_filesize );
		$presync = array(
			'max_execution_time' => $max_execution_time > 0 ? $max_execution_time : 666,
			'post_max_size' => $post_max_size,
			'upload_max_filesize' => $upload_max_filesize > 0 ? $upload_max_filesize : 66600000,
			'max_size' => $max > 0 ? $max : 66600000
		);
		$this->postprocess();
		$this->result = $presync;
		return $presync;
	}

	// Sync file
	function sync_collection( $args ) {
		global $wplr;
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		if ( !$list = $wplr->sync_collection( $args[0] ) ) {
			$this->postprocess();
			return $this->error;
		}
		$this->postprocess();
		$this->result = $list;
		return $list;
	}

	// Sync file
	function sync( $args ) {
		global $wplr;
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		$fileinfo = $args[0];
		$lrinfo = new Meow_WPLR_LRInfo();
		$lrinfo->lr_id = $fileinfo["id"];
		$lrinfo->lr_file = $fileinfo["file"];
		$lrinfo->lr_title = $fileinfo["title"];
		$lrinfo->lr_caption = $fileinfo["caption"];
		$lrinfo->lr_desc = $fileinfo["desc"];
		$lrinfo->lr_alt_text = $fileinfo["altText"];
		$lrinfo->sync_title = $fileinfo["syncTitle"];
		$lrinfo->sync_caption = $fileinfo["syncCaption"];
		$lrinfo->sync_desc = $fileinfo["syncDesc"];
		$lrinfo->sync_alt_text = $fileinfo["syncAltText"];
		$lrinfo->type = $fileinfo["type"];
		$lrinfo->tags = $fileinfo["tags"];
		$file = $wplr->b64_to_file( $fileinfo["data"] );
		if ( !isset( $fileinfo["wp_col_id"] ) || $fileinfo["wp_col_id"] == null )
			$fileinfo["wp_col_id"] = -1;
		if ( !$sync = $wplr->sync_media( $lrinfo, $file, $fileinfo["wp_col_id"] ) ) {
			$this->postprocess();
			return $this->error;
		}
		$this->postprocess();
		$this->result = $sync;
		return $sync;
	}

	// Delete file
	function delete( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		$list = $wplr->delete_media( $args[0], isset( $args[1] ) ? $args[1] : -1 );
		$this->postprocess();
		$this->result = $list;
		return $list;
	}

	// Delete collection
	function delete_collection( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		$res = $wplr->delete_collection( $args[0] );
		$this->postprocess();
		$this->result = $res;
		return $res;
	}

	// List synced files
	function list_sync_media( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		$list = $wplr->list_sync_media();
		$this->postprocess();
		$this->result = $list;
		return $list;
	}

	// List files (the Media IDs) that are not linked
	function list_unlinks( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		$list = $wplr->list_unlinks();
		$this->postprocess();
		$this->result = $list;
		return $list;
	}

	// Get LinkInfo for the Media ID
	function linkinfo( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		$linkinfo = $wplr->linkinfo_media( $args[0] );
		$this->postprocess();
		$this->result = $linkinfo;
		return $linkinfo;
	}

	// Get LinkInfo for the upload
	function linkinfo_upload( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		$file = $wplr->b64_to_file( $args[0] );
		$linkinfo = $wplr->linkinfo_upload( $file, null );
		$this->postprocess();
		$this->result = $linkinfo;
		return $linkinfo;
	}

	// Get LinkInfo for the Media ID
	function userinfo( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		$this->postprocess();
		$this->result = $user->data;
		return $user->data;
	}

	// Get LinkInfo for the Media ID
	function link( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		$lr_id = $args[0];
		$wp_id = $args[1];
		global $wplr;
		$res = $wplr->link_media( $lr_id, $wp_id );
		$this->postprocess();
		$this->result = $res;
		return $res;
	}

	// Unlink
	function unlink( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		$lr_id = $args[0];
		$wp_id = $args[1];
		global $wplr;
		$res = $wplr->unlink_media( $lr_id, $wp_id );
		$this->postprocess();
		$this->result = $res;
		return $res;
	}

	// Get WP IDs for a given LR ID
	function list_wpids( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		$lr_id = $args[0];
		global $wplr;
		$res = $wplr->list_wpids( $lr_id );
		$this->postprocess();
		$this->result = $res;
		return $res;
	}

	// Order collection
	function order_collection( $args ) {
		$args = $this->preprocess( $args );
		if ( !$user = $this->init_with( $args ) ) {
			$this->postprocess();
			return $this->error;
		}
		global $wplr;
		if ( empty( $args[0] ) || empty( $args[1] ) ) {
			return true;
		}
		$res = $wplr->order_collection( $args[0], $args[1] );
		$this->postprocess();
		$this->result = $res;
		return $res;
	}

}

?>
