<?php

class Meow_WPLR_Sync_Core {
	private $error;

	public function __construct() {
		add_action( 'delete_attachment', array( $this, 'delete_attachment' ) );
		add_filter( 'manage_media_columns', array( $this, 'manage_media_columns' ) );
		add_filter( 'shortcode_atts_gallery', array( $this, 'gallery_images_shortcode' ), 10, 3 );
		//add_filter( 'foogallery_shortcode_atts', array( $this, 'foogallery_shortcode' ), 10, 1 );
		add_action( 'manage_media_custom_column', array( $this, 'manage_media_custom_column' ), 10, 2 );
		add_action( 'admin_head', array( $this, 'admin_head' ), 10, 2 );
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
		add_action( 'init', array( $this, 'init' ) );
	}

	/*****************************************************************************
		WP GALLERY: FOLDERS, COLLECTIONS AND KEYWORDS
	*****************************************************************************/

	// function foogallery_shortcode( $atts ) {
	// 	$ids = $this->gallery_images( array(), $atts );
	// 	if ( !empty( $ids ) && count( $ids ) > 0 ) {
	// 		$atts['ids'] = $ids;
	// 	}
	// 	return $atts;
	// }

	function gallery_images_shortcode( $result, $defaults, $atts ) {
		$ids = $this->gallery_images( array(), $atts );
		if ( !empty( $ids ) && count( $ids ) > 0 ) {
			$result['include'] = $ids;
			$result['id'] = null;
			$result['order'] = '';
			$result['orderby'] = 'post__in';
		}
		return $result;
	}

	function gallery_images( $ids, $attrs ) {
		$ids = !empty( $ids ) ? $ids : array();
		if ( !empty( $attrs['wplr-collection'] ) ) {
			$collections = explode( ',', $attrs['wplr-collection'] );
			foreach ( $collections as $collection ) {
				$newIds = $this->get_media_from_collection( $collection );
				$ids = array_merge( $ids, $newIds );
			}
		}
		if ( !empty( $attrs['wplr-collections'] ) ) {
			$collections = explode( ',', $attrs['wplr-collections'] );
			foreach ( $collections as $collection ) {
				$newIds = $this->get_media_from_collection( $collection );
				$ids = array_merge( $ids, $newIds );
			}
		}
		if ( !empty( $attrs['wplr-keyword'] ) ) {
			$keywords = explode( ',', $attrs['wplr-keyword'] );
			foreach ( $keywords as $keyword ) {
				$newIds = $this->get_media_from_tag( $keyword );
				$ids = array_merge( $ids, $newIds );
			}
		}
		if ( !empty( $attrs['wplr-keywords'] ) ) {
			$keywords = explode( ',', $attrs['wplr-keywords'] );
			foreach ( $keywords as $keyword ) {
				$newIds = $this->get_media_from_tag( $keyword );
				$ids = array_merge( $ids, $newIds );
			}
		}
		return $ids;
	}

	/*****************************************************************************
		INIT
	*****************************************************************************/

	function init() {
		if ( get_option( 'wplr_enable_keywords', false ) || get_option( 'wplr_sync_keywords', false ) ) {
			include "lrsync_keywords.php";
			new Meow_WPLR_Sync_Keywords();
		}
	}

	function plugins_loaded() {
		$loaded = load_plugin_textdomain( 'wplr-sync', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		$plugins = get_option( 'wplr_plugins' );
		if ( is_array( $plugins ) ) {
			$dir = trailingslashit( plugin_dir_path( __FILE__ ) ) . trailingslashit( 'extensions' );
			$valid = array();
			$isdead = false;
			foreach ( $plugins as $plugin ) {
				if ( file_exists( trailingslashit( $dir ) . $plugin ) ) {
					include( trailingslashit( $dir ) . $plugin  );
					array_push( $valid, $plugin );
				}
				else {
					$plugin = preg_replace( '/([0-9]_)(\w+)/i', '$2', $plugin );
					if ( file_exists( trailingslashit( $dir ) . $plugin ) ) {
						include( trailingslashit( $dir ) . $plugin  );
						array_push( $valid, $plugin );
					}
					$isdead = true;
				}
			}
			if ( $isdead ) {
				update_option( 'wplr_plugins', $valid );
			}
		}
	}

	/*
		CORE
	*/

	function log( $data, $force = false ) {
		if ( !$force && !get_option( 'wplr_debuglogs', false ) )
			return;
		$fh = fopen( trailingslashit( dirname(__FILE__) ) . '/wplr-sync.log', 'a' );
		$date = date( "Y-m-d H:i:s" );
		fwrite( $fh, "$date: {$data}\n" );
		fclose( $fh );
	}

	function check_db() {
		global $wpdb;
		$tbl_m = $wpdb->prefix . 'lrsync_meta';
		if ( !$wpdb->get_var( $wpdb->prepare( "SELECT COUNT(1) FROM information_schema.tables WHERE table_schema = '%s' AND table_name = '%s';", $wpdb->dbname, $tbl_m ) ) ) {
			meow_wplrsync_activate();
		}
	}

	function reset_db() {
		do_action( 'wplr_reset' );
		meow_wplrsync_uninstall();
		meow_wplrsync_activate();
	}

	function get_error() {
		if ( $this->error ) {
			return $this->error;
		}
		else {
			return new IXR_Error( 401, __( 'Unknown error.' ) );
		}
	}

	function wpml_original_id( $wpid ) {
		if ( $this->wpml_media_is_installed() ) {
			global $sitepress;
			$language = $sitepress->get_default_language( $wpid );
			return icl_object_id( $wpid, 'attachment', true, $language );
		}
		return $wpid;
	}

	function wpml_media_is_installed() {
		return defined( 'WPML_MEDIA_VERSION' );
		//return function_exists( 'icl_object_id' ) && !class_exists( 'Polylang' );
	}

	function wpml_original_array( $wpids ) {
		if ( $this->wpml_media_is_installed() ) {
			for ($c = 0; $c < count( $wpids ); $c++ ) {
				$wpids[$c] = $this->wpml_original_id( $wpids[$c] );
			}
			$wpids = array_unique( $wpids );
		}
		return $wpids;
	}

	function get_tags_from_media( $mediaId ) {
		global $wpdb;
		$tbl_meta = $wpdb->prefix . "lrsync_meta";
		$results = $wpdb->get_col( $wpdb->prepare( "
			SELECT value
			FROM $tbl_meta
			WHERE id = %d AND name = 'media_tag'
		", $mediaId ) );
		return $results;
	}

	function get_media_from_tag( $id ) {
		global $wpdb;
		$tbl_meta = $wpdb->prefix . "lrsync_meta";
		$results = $wpdb->get_col( $wpdb->prepare( "
			SELECT id
			FROM $tbl_meta
			WHERE value = %d AND name = 'media_tag'
		", $id ) );
		return $results;
	}

	function get_media_from_collection( $id ) {
		global $wpdb;
		$tbl_relations = $wpdb->prefix . "lrsync_relations";
		$results = $wpdb->get_col( $wpdb->prepare( "
			SELECT wp_id
			FROM $tbl_relations
			WHERE wp_col_id = %d
			ORDER BY sort
		", $id ) );
		return $results;
	}

	function get_collection( $id ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync_collections";
		$info = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE wp_col_id = %d", $id ), OBJECT );
		return $info;
	}

	function get_collections_from_media( $mediaId ) {
		global $wpdb;
		$tbl_relations = $wpdb->prefix . "lrsync_relations";
		$results = $wpdb->get_col( $wpdb->prepare( "
			SELECT wp_col_id
			FROM $tbl_relations
			WHERE wp_id = %d AND wp_col_id >= 0
		", $mediaId ) );
		return $results;
	}

	function read_collections_recursively( $parent = null, $results = array(), $isRemoval = false, $level = 0 ) {
		global $wpdb;
		$tbl_c = $wpdb->prefix . 'lrsync_collections';
		if ( is_null( $parent ) )
			$collections = $wpdb->get_results( "SELECT wp_col_id, name, wp_folder_id, is_folder FROM $tbl_c WHERE wp_folder_id IS NULL ORDER BY lr_col_id", ARRAY_A );
		else
			$collections = $wpdb->get_results( $wpdb->prepare( "SELECT wp_col_id, name, wp_folder_id, is_folder FROM $tbl_c WHERE wp_folder_id = %d ORDER BY lr_col_id", $parent ), ARRAY_A );
		foreach ( $collections as $c ) {
			array_push( $results, array_merge(
				array(
					'level' => $level,
					'action' => $isRemoval ? 'remove_collection' : 'add_collection',
				), $c ) );
			if ( $c['is_folder'] )
				$results = $this->read_collections_recursively( $c['wp_col_id'], $results, $isRemoval, $level + 1 );
		}
		return $results;
	}

	function get_meta_from_value( $name, $value, $isArray = false ) {
		global $wpdb;
		$tbl_meta = $wpdb->prefix . "lrsync_meta";
		$results = $wpdb->get_col( $wpdb->prepare( "SELECT id FROM $tbl_meta WHERE name = %s AND value = %s", $name, $value ) );
		if ( count( $results ) > 1 )
			return $results;
		else if ( count( $results ) == 1 )
			return $isArray ? array( $results[0] ) : $results[0];
		return $isArray ? array() : null;
	}

	function get_meta( $name, $id, $isArray = false ) {
		global $wpdb;
		$tbl_meta = $wpdb->prefix . "lrsync_meta";
		$results = $wpdb->get_col( $wpdb->prepare( "SELECT value FROM $tbl_meta WHERE name = %s AND id = %d", $name, $id ) );
		if ( count( $results ) > 1 )
			return $results;
		else if ( count( $results ) == 1 )
			return $isArray ? array( $results[0] ) : $results[0];
		return $isArray ? array() : null;
	}

	function set_meta( $name, $id, $value, $unique = false ) {
		global $wpdb;
		$tbl_meta = $wpdb->prefix . "lrsync_meta";
		if ( $unique ) {
			$count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $tbl_meta WHERE name = %s AND id = %d", $name, $id ) );
			if ( $count > 0 ) {
				$wpdb->update( $tbl_meta, array( 'value' => $value ), array( 'name' => $name, 'id' => $id ), array( '%s' ), array( '%s', '%d' ) );
				return true;
			}
		}
		$wpdb->insert( $tbl_meta, array( 'name' => $name, 'id' => $id, 'value' => $value ) );
	}

	// If no value given, all meta for this id will be deleted
	function delete_meta( $name, $id, $value = null ) {
		global $wpdb;
		$tbl_meta = $wpdb->prefix . "lrsync_meta";
		if ( is_null( $value ) )
			$wpdb->query( $wpdb->prepare( "DELETE FROM $tbl_meta WHERE name = %s AND id = %d", $name, $id ) );
		else
			$wpdb->query( $wpdb->prepare( "DELETE FROM $tbl_meta WHERE name = %s AND id = %d AND value = %s", $name, $id, $value ) );
	}

	// Return SyncInfo for this WP ID
	function get_sync_info( $wpid ) {
		$wpid = $this->wpml_original_id( $wpid );
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$info = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE wp_id = %d", $wpid ), OBJECT );
		return $info;
	}

	function get_sync_info_from_lr_id( $lr_id ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$info = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE lr_id = %d", $lr_id ), OBJECT );
		return $info;
	}

	function get_hierarchy( $parent = null, $level = 0 ) {
		global $wpdb;
		$tbl_m = $wpdb->prefix . 'lrsync';
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$tbl_c = $wpdb->prefix . 'lrsync_collections';

		$current = array();

		if ( is_null( $parent ) )
			$collections = $wpdb->get_results( "SELECT * FROM $tbl_c WHERE wp_folder_id IS NULL ORDER BY lr_col_id", OBJECT );
		else
			$collections = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $tbl_c WHERE wp_folder_id = %d ORDER BY lr_col_id", $parent ), OBJECT );

		foreach ( $collections as $c ) {
			$photos_count = null;
			if ( !$c->is_folder )
				$photos_count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $tbl_r WHERE wp_col_id = %d", $c->wp_col_id ), OBJECT );
			$current[] = array(
				'id' => $c->wp_col_id,
				'level' => $level,
				'type' => $c->is_folder ? 'folder' : 'collection',
				'name' => $c->name,
				'count' => $photos_count,
				'children' => $c->is_folder ? $this->get_hierarchy( $c->wp_col_id, $level + 1 ) : null
			);
		}
		return $current;
	}

	function get_keywords_hierarchy( $parent = null, $level = 0 ) {
		global $wpdb;
		$tbl_m = $wpdb->prefix . 'lrsync_meta';

		$current = array();

		if ( is_null( $parent ) )
			$collections = $wpdb->get_results( "SELECT meta_id,
				MAX(IF(`name` = 'tag_name', id, NULL)) id,
				MAX(IF(`name` = 'tag_name', value, NULL)) name,
				MAX(IF(`name` = 'tag_parent', value, NULL)) parent
				FROM $tbl_m
				WHERE name = 'tag_name' OR name = 'tag_parent'
				GROUP BY id
				HAVING parent IS NULL
				ORDER BY id", OBJECT );
		else {
			$collections = $wpdb->get_results( $wpdb->prepare( "SELECT meta_id,
				MAX(IF(`name` = 'tag_name', id, NULL)) id,
				MAX(IF(`name` = 'tag_name', value, NULL)) name,
				MAX(IF(`name` = 'tag_parent', value, NULL)) parent
				FROM $tbl_m
				WHERE name = 'tag_name' OR name = 'tag_parent'
				GROUP BY id
				HAVING parent = %d
				ORDER BY id", $parent  ), OBJECT );
		}
		foreach ( $collections as $c ) {
			$photos_count = null;
			$count = $wpdb->get_var( "SELECT COUNT(*) FROM $tbl_m WHERE name = 'media_tag' AND value = {$c->id}" );
			$current[] = array(
				'id' => $c->id,
				'level' => $level,
				'name' => $c->name,
				'count' => $count,
				'children' => !empty( $c->id ) ? $this->get_keywords_hierarchy( $c->id, $level + 1 ) : array()
			);
		}
		return $current;
	}

	function get_gallery( $id ) {
		global $wpdb;
		$tbl_m = $wpdb->prefix . 'lrsync';
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$tbl_c = $wpdb->prefix . 'lrsync_collections';
		$tbl_meta = $wpdb->prefix . 'postmeta';
		$photos = $wpdb->get_results( $wpdb->prepare( "
			SELECT wp_id id
			FROM $tbl_r r
			WHERE wp_col_id = %d
			ORDER BY sort", $id ), ARRAY_A );
		foreach ( $photos as &$photo ) {
			$id = $photo['id'];
			$photo['title'] = get_the_title( $id );
			$photo['full_size'] = stripslashes( wp_get_attachment_url( $id ) );
			$photo['thumbnail'] = stripslashes( wp_get_attachment_thumb_url( $id, 'post-thumbnail' ) );
		}
		return $photos;
	}

	function delete_media( $lr_id, $wp_col_id = null ) {
		global $wpdb;
		$lrinfo = $this->get_sync_info_from_lr_id( $lr_id );

		if ( empty( $lrinfo ) ) {
			error_log( "WP/LR Sync: seems like this media doesn't exist or has been already removed ($lr_id)." );
			return true;
		}

		// Delete relations in in collections
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		if ( !is_null( $wp_col_id ) ) {
			$wpdb->query( $wpdb->prepare( "DELETE FROM $tbl_r WHERE wp_id = %d AND wp_col_id = %d", $lrinfo->wp_id, $wp_col_id ) );
			if ( $wp_col_id >= 0 )
				do_action( 'wplr_remove_media_from_collection', (int)$lrinfo->wp_id, (int)$wp_col_id );
		}

		// Delete media if it is not part of any collection
		$left = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $tbl_r WHERE wp_id = %d", $lrinfo->wp_id ) );
		if ( $left < 1 ) {
			// Delete the media, it is not used anywhere
			$table_name = $wpdb->prefix . "lrsync";
			$sync_files = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $table_name WHERE lr_id = %d", $lr_id), OBJECT );
			$delete_count = 0;
			foreach ( $sync_files as $sync ) {
				if ( wp_delete_attachment( $sync->wp_id, true ) ) {
					$wpdb->query( $wpdb->prepare( "DELETE FROM $table_name WHERE lr_id = %d", $lr_id ) );
					$delete_count++;

					//TODO: DELETE KEYWORDS
					// Are the tags used by this Media now useless?
					$keywords = $this->get_meta( 'media_tag', $sync->wp_id, true );
					foreach ( $keywords as $keyword ) {
						$results = $this->get_meta_from_value( 'media_tag', $keyword, true );
						if ( count( $results ) < 1 ) {
							$this->delete_keyword( $keyword );
						}
					}

				}
			}
			if ( count( $sync_files ) < 1 ) {
				// There were no files to remove
				return true;
			}
			else if ( $delete_count > 0 ) {
				// Files were removed
				do_action( 'wplr_remove_media', (int)$lrinfo->wp_id );
				return true;
			}
			else {
				// Nothing was removed, strangely
				$this->error = new IXR_Error( 403, __( "The attachment could not be removed." ) );
				return false;
			}
		}
		else {
			// Don't delete the media, it is used somewhere else
			return true;
		}
	}

	function delete_attachment( $wp_id ) {
		$wp_id = $this->wpml_original_id( $wp_id );
		$this->sync_media_tags( $wp_id );
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$sql = $wpdb->prepare( "DELETE FROM $table_name WHERE wp_id = %d", $wp_id );
		$wpdb->query( $sql );
		$table_name = $wpdb->prefix . "lrsync_relations";
		$sql = $wpdb->prepare( "DELETE FROM $table_name WHERE wp_id = %d", $wp_id );
		$wpdb->query( $sql );
	}

	function unlink_media( $lr_id, $wp_id ) {
		$wp_id = $this->wpml_original_id( $wp_id );
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";

		// Remove media
		if ( $wp_id ) {
			$sync = $this->get_sync_info( $wp_id );
			if ( $sync ) {
				$wpdb->query( $wpdb->prepare( "DELETE FROM $table_name WHERE wp_id = %d", $sync->wp_id ) );
				return true;
			}
		}
		else {
			$wpdb->query( $wpdb->prepare( "DELETE FROM $table_name WHERE lr_id = %d", $sync->lr_id ) );
			return true;
		}

		$this->error = new IXR_Error( 403, __( "There is no link for this media." ) );
		return false;
	}

	function link_media( $lr_id, $wp_id ) {
		$wp_id = $this->wpml_original_id( $wp_id );
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		if ( empty( $wp_id ) ) {
			$this->error = new IXR_Error( 403, __( "The arguments lr_id and wp_id are required." ) );
			return false;
		}
		if ( !wp_attachment_is_image( $wp_id ) ) {
			$this->error = new IXR_Error( 403, __( "Attachment " . ($wp_id ? $wp_id : "[null]") . " does not exist or is not an image." ) );
			return false;
		}
		$sync = $this->get_sync_info( $wp_id );
		if ( !$sync ) {
			$wpdb->insert( $table_name,
				array(
					'wp_id' => $wp_id,
					'lr_id' => $lr_id,
					'lr_file' => null,
					'lastsync' => null
				)
			);
		}
		else {
			$wpdb->query( $wpdb->prepare( "UPDATE $table_name
				SET lr_id = %d
				WHERE wp_id = %d", $lr_id, $wp_id )
			);
		}
		$sync = $this->get_sync_info( $wp_id );
		$info = Meow_WPLR_LRInfo::fromRow( $sync );
		return $info;
	}

	function list_sync_media() {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$sync_files = $wpdb->get_results( "SELECT * FROM $table_name WHERE lr_id >= 0 AND wp_id >= 0", OBJECT );
		$list = array();
		foreach ( $sync_files as $sync_file ) {
			$info = Meow_WPLR_LRInfo::fromRow( $sync_file );
			array_push( $list, $info );
		}
		return $list;
	}

	function update_metadata( $wp_id, $lrinfo, $isTranslation = false ) {
		// Update Title, Description and Caption

		$meta = null;
		if ( $isTranslation ) {
			$meta = get_post( $wp_id, ARRAY_A );
		}

		// Update Title, Caption and Desc (if needed)
		if ( $lrinfo->sync_title || $lrinfo->sync_caption || $lrinfo->sync_desc ) {
			$post = array( 'ID' => $wp_id );
			if ( $lrinfo->sync_title && ( !$meta || empty( $meta['post_title'] ) ) )
				$post['post_title'] = $lrinfo->lr_title;
			if ( $lrinfo->sync_desc && ( !$meta || empty( $meta['post_content'] ) ) )
			if ( $lrinfo->sync_caption && ( !$meta || empty( $meta['post_excerpt'] ) ) )
				$post['post_content'] = $lrinfo->lr_desc;
				$post['post_excerpt'] = $lrinfo->lr_caption;
			wp_update_post( $post );
		}

		// Update Alt Text if needed
		if ( $lrinfo->sync_alt_text ) {
			if ( $isTranslation )
				$meta_alt = get_post_meta( $wp_id, '_wp_attachment_image_alt', true );
			if ( !$isTranslation || empty( $meta_alt ) )
				update_post_meta( $wp_id, '_wp_attachment_image_alt', $lrinfo->lr_alt_text );
		}
	}

	function create_keyword( $lrTagId, $name, $lrTagParentId = null ) {
		$this->set_meta( 'tag_name', $lrTagId, $name, true );
		$this->set_meta( 'tag_parent', $lrTagId, $lrTagParentId, true );
		do_action( 'wplr_add_tag', (int)$lrTagId, $name, $lrTagParentId );
	}

	function update_keyword( $lrTagId, $name ) {
		$this->set_meta( 'tag_name', $lrTagId, $name, true );
		do_action( 'wplr_update_tag', (int)$lrTagId, $name );
	}

	function move_keyword( $lrTagId, $lrTagParentId ) {
		$previous = $this->get_meta( 'tag_parent', $lrTagId );
		$this->set_meta( 'tag_parent', $lrTagId, $lrTagParentId, true );
		do_action( 'wplr_move_tag', (int)$lrTagId, $lrTagParentId, $previous );
	}

	function delete_keyword( $lrTagId ) {
		$this->delete_meta( 'tag_name', $lrTagId );
		$this->delete_meta( 'tag_parent', $lrTagId );
		do_action( 'wplr_remove_tag', (int)$lrTagId );
		$kids = $this->get_meta_from_value( 'tag_parent', $lrTagId, true );
		foreach ( $kids as $kid ) {
			$kidName = $this->get_meta( 'tag_name', $kid );
			$this->delete_meta( 'tag_parent', $kid );
			do_action( 'wplr_update_tag', (int)$kid, $kidName, null );
		}
		$media = $this->get_meta_from_value( 'media_tag', $lrTagId, true );
		foreach ( $media as $m ) {
			$this->delete_meta( 'media_tag', $m );
			do_action( 'wplr_remove_media_tag', (int)$m, $lrTagId );
		}
	}


	function flatten_tags( $importTags, $allTags = array() ) {
		foreach ( $importTags as $tag ) {
			$parentId = null;
			$currentTag = $tag;
			$allTags[(int)$tag['id']] = array(
				'id' => $tag['id'],
				'name' => trim( $tag['name'], '\\"\' '),
				'parent' => null
			);
			while ( isset( $currentTag['parent'] ) && is_array( $currentTag['parent'] ) ) {
				$allTags[(int)$currentTag['id']]['parent'] = $currentTag['parent']['id'];
				$currentTag = $currentTag['parent'];
				$allTags[(int)$currentTag['id']] = array(
					'id' => $currentTag['id'],
					'name' => trim( $currentTag['name'], '\\"\' '),
					'parent' => null
				);
			}
		}
		return $allTags;
	}

	function sync_media_tags( $wp_id, $tags = '' ) {
		// If tags is not an array (so maybe an old string of tags, or empty tag, let's set it to empty)
		if ( !is_array( $tags ) )
			$tags = array();
		$newTags = array();
		$flatten = $this->flatten_tags( $tags );
		$deathcount = 1666;

		// Read the tags given by LR, add them in the meta if they are new.
		while ( count( $flatten ) > 0 && $deathcount > 0  ) {
			$tag = array_shift( $flatten );

			$deathcount--;
			$pTagName = $this->get_meta( 'tag_name', $tag['id'] );
			$pTagParent = $this->get_meta( 'tag_parent', $tag['id'] );

			// Tag does not exist
			if ( empty( $pTagName ) ) {

				// Has no parent, we can create it
				if ( $tag['parent'] == null ) {
					$this->create_keyword( $tag['id'], $tag['name'] );
					array_push( $newTags, $tag['id'] );
					continue;
				}
				// Has a parent, which is already registered
				$parentName = $this->get_meta( 'tag_name', $tag['parent'] );
				if ( !empty( $parentName ) ) {
					$this->create_keyword( $tag['id'], $tag['name'], $tag['parent'] );
					array_push( $newTags, $tag['id'] );
					continue;
				}
			}

			// Tag exists
			if ( !empty( $pTagName ) ) {

				// But has different name, so we update it
				if ( $pTagName != $tag['name'] )
					$this->update_keyword( $tag['id'], $tag['name'] );

				// But has different parent
				if ( $pTagParent != (int)$tag['parent'] )
					$this->move_keyword( $tag['id'], (int)$tag['parent'] );

				array_push( $newTags, $tag['id'] );
				continue;
			}

			// Couldn't handle the tag, we put it back in
			array_push( $flatten, $tag );
		}

		// Take care of the tags for the media
		$oldTags = $this->get_meta( 'media_tag', $wp_id, true );
		if ( !is_array( $oldTags ) )
			$oldTags = array();
		$toAdds = array_diff( $newTags, $oldTags );
		foreach ( $toAdds as $toAdd ) {
			$this->set_meta( 'media_tag', $wp_id, $toAdd );
			do_action( 'wplr_add_media_tag', (int)$wp_id, trim( $toAdd ) );
		}
		$toDeletes = array_diff( $oldTags, $newTags );
		if ( count( $toDeletes > 0 ) ) {
			foreach ( $toDeletes as $toDelete ) {
				$this->delete_meta( 'media_tag', $wp_id, $toDelete );
				do_action( 'wplr_remove_media_tag', (int)$wp_id, trim( $toDelete ) );

				// Is the tag now useless?
				$results = $this->get_meta_from_value( 'media_tag', $toDelete, true );
				if ( count( $results ) < 1 ) {
					$this->delete_keyword( $toDelete );
				}
			}
		}
		return true;
	}

	//If user prefers to use the time of the image instead of "now" in Media Library
	function update_media_date( $wp_id ) {
		if ( get_option( 'wplr_use_taken_date', false ) && function_exists( 'exif_read_data' ) ) {
			$current_file = get_attached_file( $wp_id );
			$exif_data = null;
			try {
				$exif_data = @exif_read_data( $current_file );
			}
			catch ( Exception $e ) {
				$exif_data = null;
				error_log( $e->getMessage() );
			}
			if ( !empty( $exif_data ) && !empty( $exif_data[ 'DateTimeOriginal' ] ) ) {
				$takentime = strtotime( $exif_data[ 'DateTimeOriginal' ] );
				$takentime = date( 'Y-m-d H:i:s', $takentime );
				$media = array(
			      'ID' => $wp_id,
			      'post_date' => $takentime,
			      'post_date_gmt' => $takentime,
			  );
				wp_update_post( $media );
			}
			else {
				$this->log( "Couldn't read the EXIF DateTimeOriginal for Media $wp_id." );
			}
		}
	}

	function sync_media_update( $lrinfo, $tmp_path, $sync ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$wp_id = $sync->wp_id;
		$meta = wp_get_attachment_metadata( $wp_id );
		$current_file = get_attached_file( $wp_id );

		// Support for WP Retina 2x
		if ( function_exists( 'wr2x_generate_images' ) )
			wr2x_delete_attachment( $wp_id );

		// The file doesn't exist anymore for some reason
		if ( !file_exists( $current_file ) ) {
			error_log( "WP/LR Sync: get_attached_file() returned empty. Assuming broken DB, delete link and continue." );
			$this->delete_attachment( $wp_id );
		}

		$pathinfo = pathinfo( $current_file );
		if ( !isset( $pathinfo['dirname'] ) ) {
			error_log( "WP/LR Sync: pathinfo() failed in sync_media_update with " . $current_file );
			$this->error = new IXR_Error( 403, __( "Could not handle the file on the server-side." ) );
			return false;
		}
		$basepath = $pathinfo['dirname'];

		// Let's clean everything first
		if ( wp_attachment_is_image( $wp_id ) ) {
			$sizes = $this->get_image_sizes();
			foreach ($sizes as $name => $attr) {
				if (isset($meta['sizes'][$name]) && isset($meta['sizes'][$name]['file']) && file_exists( trailingslashit( $basepath ) . $meta['sizes'][$name]['file'] )) {
					$normal_file = trailingslashit( $basepath ) . $meta['sizes'][$name]['file'];
					$pathinfo = pathinfo( $normal_file );

					// Support for WP Retina 2x
					if ( function_exists( 'wr2x_generate_images' ) )
						$retina_file = trailingslashit( $pathinfo['dirname'] ) . $pathinfo['filename'] . wr2x_retina_extension() . $pathinfo['extension'];

					// Test if the file exists and if it is actually a file (and not a dir)
					// Some old WordPress Media Library are sometimes broken and link to directories
					if ( file_exists( $normal_file ) && is_file( $normal_file ) )
						unlink( $normal_file );

					// Support for WP Retina 2x
					if ( function_exists( 'wr2x_generate_images' ) && ( file_exists( $retina_file ) && is_file( $retina_file ) ) )
							unlink( $retina_file );
				}
			}
		}
		if ( file_exists( $current_file ) )
			unlink( $current_file );

		// Insert the new file and delete the temporary one
		copy( $tmp_path, $current_file );
		chmod( $current_file, 0644 );

		// Update the Upload/TakenTime Date
		$this->update_media_date( $wp_id );

		// Update metadata
		$this->update_metadata( $wp_id, $lrinfo );

		// If there are translations, maybe they need to be updated too!
		// Udate 2017/08/28: No, it's better to only keep the main media translated.
		// if ( $this->wpml_media_is_installed() ) {
		// 	global $sitepress;
		// 	$trid = $sitepress->get_element_trid( $wp_id, 'post_attachment' );
		// 	$translations = $sitepress->get_element_translations( $trid, 'post_attachment' );
		// 	foreach( $translations as $k => $v ) {
		// 		if ( $v->element_id != $wp_id )
		// 			$this->update_metadata( $v->element_id, $lrinfo, true );
		// 	}
		// }

		// Generate the images
		require_once( ABSPATH . 'wp-admin/includes/image.php' );
		$metadata = wp_generate_attachment_metadata( $wp_id, $current_file );
		wp_update_attachment_metadata( $wp_id, $metadata );

		// Support for WP Retina 2x
		if ( function_exists( 'wr2x_generate_images' ) )
			wr2x_generate_images( wp_get_attachment_metadata( $wp_id ) );

		$wpdb->query( $wpdb->prepare( "UPDATE $table_name
			SET lr_file = %s, lastsync = NOW()
			WHERE lr_id = %d", $lrinfo->lr_file, $lrinfo->lr_id )
		);

		if ( !empty( $lrinfo->tags ) )
			$this->sync_media_tags( $wp_id, $lrinfo->tags );

		$tbl_r = $wpdb->prefix . "lrsync_relations";
		$gallery_ids = $wpdb->get_col( $wpdb->prepare( "SELECT wp_col_id FROM $tbl_r WHERE wp_id = %d", $wp_id ) );
		do_action( 'wplr_update_media', (int)$wp_id, $gallery_ids );

		return true;
	}

	function wplr_sanitize_filename( $filename ) {
		if ( get_option( 'wplr_filename_accents', false ) )
			return $filename;
		$path = pathinfo( $filename );
		$new = preg_replace( '/.' . $path['extension'] . '$/', '', $filename );
		return sanitize_title( $new ) . '.' . $path['extension'];
	}

	function sync_media_add( $lrinfo, $tmp_path ) {
		global $wpdb;
		$tbl_wplr = $wpdb->prefix . "lrsync";
		$upload_dir = wp_upload_dir();
		$newfile = wp_unique_filename( $upload_dir["path"], $this->wplr_sanitize_filename( $lrinfo->lr_file ) );
		$newpath = trailingslashit( $upload_dir["path"] ) . $newfile;
		if ( !@rename( $tmp_path, $newpath ) ) {
			$this->error = new IXR_Error( 403, __( "Could not copy the file." ) );
			return false;
		}
		$wp_upload_dir = wp_upload_dir();
		if ( !$wp_id = wp_insert_attachment( array(
			'guid' => $wp_upload_dir['url'] . '/' . basename( $newpath ),
			'post_title' => $lrinfo->lr_title,
			'post_content' => $lrinfo->lr_desc,
			'post_excerpt' => $lrinfo->lr_caption,
			'post_mime_type' => $lrinfo->type,
			'post_status' => "inherit",
		), $newpath ) ) {
			$this->error = new IXR_Error( 403, __( "Could not insert attachment for " . $newpath ) );
			return false;
		}

		// Create Alt Text
		update_post_meta( $wp_id, '_wp_attachment_image_alt', $lrinfo->lr_alt_text );

		require_once( ABSPATH . 'wp-admin/includes/image.php' );
		$attach_data = wp_generate_attachment_metadata( $wp_id, $newpath );
		wp_update_attachment_metadata( $wp_id, $attach_data );

		// Support for WP Retina 2x
		if ( function_exists( 'wr2x_generate_images' ) ) {
			wr2x_generate_images( $attach_data );
		}

		$wpdb->insert( $tbl_wplr,
			array(
				'wp_id' => $wp_id,
				'lr_id' => ( $lrinfo->lr_id == "" || $lrinfo->lr_id == null ) ? -1 : $lrinfo->lr_id,
				'lr_file' => $lrinfo->lr_file,
				'lastsync' => current_time( 'mysql' )
			)
		);

		// Update the Upload/TakenTime Date
		$this->update_media_date( $wp_id );

		do_action( 'wplr_add_media', (int)$wp_id );

		if ( !empty( $lrinfo->tags ) )
			$this->sync_media_tags( $wp_id, $lrinfo->tags );

		return true;
	}

	function sync_media( $lrinfo, $tmp_path, $wp_col_id = null ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$sync_files = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $table_name WHERE lr_id = %d", $lrinfo->lr_id ), OBJECT );

		if ( $tmp_path == null || empty( $tmp_path ) ) {
			$this->error = new IXR_Error( 403, __( "The file was not uploaded." ) );
			return false;
		}

		// Never synced, create the attachment
		if ( !$sync_files ) {
			if ( !$this->sync_media_add( $lrinfo, $tmp_path ) )
				return false;
		}

		// Synced info found in DB, go through them
		else {
			$updates = 0;
			foreach ( $sync_files as $sync ) {
				if ( $this->sync_media_update( $lrinfo, $tmp_path, $sync ) )
					$updates++;
			}
			// In case DB is broken and no updates was made, we need to create the attachment
			if ( $updates == 0 ) {
				if ( !$this->sync_media_add( $lrinfo, $tmp_path ) )
					return false;
			}
		}
		if ( file_exists( $tmp_path ) )
			unlink( $tmp_path );

		// Returns only one result even if there are many.
		$sync = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE lr_id = %d", $lrinfo->lr_id ), OBJECT );
		$info = Meow_WPLR_LRInfo::fromRow( $sync );
		if ( empty( $info ) ) {
			$this->error = new IXR_Error( 403, __( "The information about the media could not be retrieved." ) );
			return false;
		}

		// Handle the collection if any
		if ( !is_null( $wp_col_id ) ) {
			$this->add_media_to_collection( $sync, $wp_col_id );
		}

		do_action( 'wplr_sync_media', $sync );
		return $info;
	}

	function get_image_sizes() {
		$sizes = array();
		global $_wp_additional_image_sizes;
		foreach (get_intermediate_image_sizes() as $s) {
			$crop = false;
			if (isset($_wp_additional_image_sizes[$s])) {
				$width = intval($_wp_additional_image_sizes[$s]['width']);
				$height = intval($_wp_additional_image_sizes[$s]['height']);
				$crop = $_wp_additional_image_sizes[$s]['crop'];
			} else {
				$width = get_option($s.'_size_w');
				$height = get_option($s.'_size_h');
				$crop = get_option($s.'_crop');
			}
			$sizes[$s] = array( 'width' => $width, 'height' => $height, 'crop' => $crop );
		}
		return $sizes;
	}

	// Returns link info for a file at this path
	function linkinfo_upload( $path, $meta = null ) {
		$I = PHasher::Instance();
		$exif = null;
		if ( $meta == null) {
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
			$meta = wp_read_image_metadata( $path );
			$exif = ( isset( $meta, $meta["created_timestamp"] ) && (int)$meta["created_timestamp"] > 0 ) ? date( "Y/m/d H:i:s", $meta["created_timestamp"] ) : null;
		}
		else if ( isset( $meta, $meta["image_meta"], $meta["image_meta"]["created_timestamp"] ) && (int)$meta["image_meta"]["created_timestamp"] > 0 ) {
			$exif = date( "Y/m/d H:i:s", $meta["image_meta"]["created_timestamp"] );
		}
		return array(
			'wp_phash' => $I->HashAsString( $I->HashImage( $path, 0, 0, 16 ), true ),
			'wp_exif' => $exif
		);
	}

	// Returns link info to help LR to find the original image
	function linkinfo_media( $wp_id ) {
		$wp_id = $this->wpml_original_id( $wp_id );
		if ( !wp_attachment_is_image( $wp_id ) ) {
			$this->error = new IXR_Error( 403, __( "Attachment " . ($wp_id ? $wp_id : "[null]") . " does not exist or is not an image." ) );
			return false;
		}
		$linkinfo = $this->linkinfo_upload( get_attached_file( $wp_id ), wp_get_attachment_metadata( $wp_id ) );
		return array(
			'wp_id' => $wp_id,
			'wp_url' => wp_get_attachment_url( $wp_id ),
			'wp_phash' => $linkinfo["wp_phash"],
			'wp_exif' => $linkinfo["wp_exif"]
		);
	}

	// Returns an array of wp_id linked to this lr_id
	function list_wpids( $lr_id ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$wp_ids = $wpdb->get_results( $wpdb->prepare( "SELECT p.wp_id FROM $table_name p WHERE p.lr_id = %d", $lr_id ) );
		return $wp_ids;
	}

	function list_ignored() {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$posts = $wpdb->get_results(
			"SELECT wp_id ID
			FROM $table_name
			WHERE lr_id = 0
			ORDER BY wp_id DESC", OBJECT );
		return $posts;
	}

	function list_duplicates() {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$images = $wpdb->get_results(
			"SELECT lr.lr_id, lr.lr_file, GROUP_CONCAT(lr.wp_id SEPARATOR ',') as wpids
			FROM $wpdb->posts p, $table_name lr
			WHERE p.ID = lr.wp_id AND lr.lr_id != 0
			GROUP BY lr.lr_id
			HAVING COUNT(p.ID) > 1
			ORDER BY lr.lr_id DESC", OBJECT );
		return $images;
	}

	function list_unlinks( $allfields = false ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		$potentials = array();

		$whereIsOriginal = "";
		if ( $this->wpml_media_is_installed() ) {
			global $sitepress;
			$tbl_wpml = $wpdb->prefix . "icl_translations";
			$language = $sitepress->get_default_language();
			$whereIsOriginal = "AND p.ID IN (SELECT element_id FROM $tbl_wpml WHERE element_type = 'post_attachment' AND language_code = '$language') ";
		}

		if ( $allfields ) {
			$posts = $wpdb->get_results( "SELECT * FROM $wpdb->posts p
			WHERE post_status = 'inherit'
			AND post_mime_type = 'image/jpeg'
			AND p.ID NOT IN (SELECT wp_id FROM $table_name) " .
			$whereIsOriginal .
			"ORDER BY p.ID DESC" );
		}
		else {
			$posts = $wpdb->get_col( "SELECT p.ID FROM $wpdb->posts p
			WHERE post_status = 'inherit'
			AND post_mime_type = 'image/jpeg'
			AND p.ID NOT IN (SELECT wp_id FROM $table_name) " .
			$whereIsOriginal .
			"ORDER BY p.ID DESC" );
		}

		foreach ( $posts as $post ) {
			if ( $allfields ) {
				if ( !wp_attachment_is_image( $post->ID ) )
					continue;
				array_push( $potentials, $post );
			}
			else {
				if ( !wp_attachment_is_image( $post ) )
					continue;
				array_push( $potentials, $post );
			}
		}
		return $potentials;
	}

	/*****************************************************************************
		COLLECTIONS
	*****************************************************************************/

	function add_media_to_collection( $lrinfo, $wp_col_id, $sort = 0 ) {
		global $wpdb;
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$tbl_col = $wpdb->prefix . 'lrsync_collections';
		$count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $tbl_r WHERE wp_id = %d AND wp_col_id = %d", $lrinfo->wp_id, $wp_col_id ) );
		if ( $count < 1 ) {
			if ( $wpdb->insert( $tbl_r,
				array( 'wp_col_id' => $wp_col_id, 'wp_id' => $lrinfo->wp_id, 'sort' => $sort ),
				array( '%d', '%d', '%s' )
			) ) {
				$wpdb->query( $wpdb->prepare( "UPDATE $tbl_col SET lastsync = %s WHERE wp_col_id = %d", current_time( 'mysql' ), $wp_col_id ) );
				if ( $wp_col_id > -1 )
					do_action( "wplr_add_media_to_collection", (int)$lrinfo->wp_id, (int)$wp_col_id, false );
			}
			else {
				$this->error = new IXR_Error( 403, __( "Could not add media " . ( isset( $lrinfo->wp_id ) ? $lrinfo->wp_id : "[null]") . " to collection " . ( isset( $wp_col_id ) ? $wp_col_id : "[null]") . "." ) );
				return false;
			}
		}
	}

	function sync_collection( $collections ) {
		global $wpdb;
		$folder = null;
		$tbl_col = $wpdb->prefix . 'lrsync_collections';
		foreach ( $collections as &$collection ) {
			$type = $collection['type'] == 'folder' ? 'folder' : 'collection';
			$parent_folder = ( !$folder || empty( $folder['wp_col_id'] ) ) ? null : (int)$folder['wp_col_id'];
			$row = empty( $collection['wp_col_id'] ) ? null : $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $tbl_col c
				WHERE wp_col_id = %d AND lr_col_id = %d", $collection['wp_col_id'], $collection['lr_col_id'] ), OBJECT );
			if ( empty( $row ) ) {
				// Collection needs to be created in DB.
				if ( $wpdb->insert( $tbl_col,
					array(
						'lr_col_id' => $collection['lr_col_id'],
						'is_folder' => $type == 'folder' ? 1 : 0,
						'name' => $collection['name'],
						'lastsync' => current_time( 'mysql' )
					),
					array( '%d', '%d', '%s', '%s' )
				) ) {
					$collection['wp_col_id'] = $wpdb->insert_id;
					if ( !is_null( $parent_folder ) ) {
						$wpdb->query( $wpdb->prepare( "UPDATE $tbl_col
							SET wp_folder_id = %s
							WHERE wp_col_id = %d", $parent_folder, $collection['wp_col_id'] )
						);
					}
				}
				else {
					// Folder/Collection failed.
					$this->error = new IXR_Error( 403, __( "Could not create the folder/collection called " . ( isset( $collection['name'] ) ? $collection['name'] : "[null]") . "." ) );
					return false;
				}
				do_action( "wplr_create_$type", (int)$collection['wp_col_id'], (int)$parent_folder, array( 'name' => $collection['name'] ) );
			}
			else {
				// Collection exists in DB, check for changes.
				$collection['wp_col_id'] = $row->wp_col_id;
				if ( $collection['name'] != $row->name ) {
					$wpdb->query( $wpdb->prepare( "UPDATE $tbl_col
						SET name = %s
						WHERE wp_col_id = %d", $collection['name'], $row->wp_col_id )
					);
					do_action( "wplr_update_$type", (int)$row->wp_col_id, array( 'name' => $collection['name'] ) );
				}
				if ( $parent_folder != $row->wp_folder_id ) {
					if ( is_null( $parent_folder ) )
						$wpdb->query( $wpdb->prepare( "UPDATE $tbl_col SET wp_folder_id = NULL WHERE wp_col_id = %d", $row->wp_col_id ) );
					else
						$wpdb->query( $wpdb->prepare( "UPDATE $tbl_col SET wp_folder_id = %d WHERE wp_col_id = %d", $parent_folder, $row->wp_col_id ) );
					do_action( "wplr_move_$type", (int)$row->wp_col_id, (int)$parent_folder, (int)$row->wp_folder_id );
				}
			}
			$folder = $collection;
		}
		return $collections;
	}

	private $maxdepth = 100;
	private $currentdepth = 0;

	function delete_collection_recursively( $tbl_col, $tbl_r, $tbl_lr, $wp_col_id ) {
		global $wpdb;
		if ( $this->currentdepth++ > $this->maxdepth ) {
			error_log( "WP/LR Sync: delete_collection_recursively() reached maxdepth." );
			return false;
		}
		$children = $wpdb->get_col( $wpdb->prepare( "SELECT DISTINCT(wp_col_id) FROM $tbl_col WHERE wp_folder_id = %d", $wp_col_id ) );
		foreach ( $children as $kid ) {
			$res = $this->delete_collection_recursively( $tbl_col, $tbl_r, $tbl_lr, $kid );
			if ( !$res )
				return false;
		}
		$lr_ids = $wpdb->get_col( $wpdb->prepare( "SELECT DISTINCT(lr_id) FROM $tbl_r r
			INNER JOIN $tbl_lr l ON r.wp_id = l.wp_id WHERE r.wp_col_id = %d", $wp_col_id ) );
		if ( !empty( $lr_ids ) ) {
			foreach ( $lr_ids as $lr_id ) {
				if ( !$this->delete_media( $lr_id, $wp_col_id ) ) {
					return false;
				}
			}
		}
		$collection = $wpdb->get_row( $wpdb->prepare( "SELECT wp_col_id, is_folder FROM $tbl_col
			WHERE wp_col_id = %d", $wp_col_id ), OBJECT, 0 );
		if ( !empty( $collection ) ) {
			$type = $collection->is_folder ? 'folder' : 'collection';
			do_action( "wplr_remove_{$type}", (int)$wp_col_id );
			$wpdb->query( $wpdb->prepare( "DELETE FROM $tbl_col WHERE wp_col_id = %d", $wp_col_id ) );
		}
		return true;
	}

	function delete_collection( $wp_col_id ) {
		global $wpdb;
		$folder = null;
		$tbl_col = $wpdb->prefix . 'lrsync_collections';
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$tbl_lr = $wpdb->prefix . 'lrsync';
		$currentdepth = 0;
		$result = $this->delete_collection_recursively( $tbl_col, $tbl_r, $tbl_lr, $wp_col_id );
		return $result;
	}

	function order_collection( $wp_col_id, $lr_ids ) {
		if ( empty( $wp_col_id ) || empty( $lr_ids ) )
			return true;
		global $wpdb;
		$tbl_r = $wpdb->prefix . 'lrsync_relations';
		$count = 0;
		foreach ( $lr_ids as $lr_id ) {
			$info = $this->get_sync_info_from_lr_id( $lr_id );
			if ( !empty( $info ) ) {
				$wpdb->query( $wpdb->prepare( "UPDATE $tbl_r SET sort = %d WHERE wp_col_id = %d AND wp_id = %d", $count, $wp_col_id, $info->wp_id ) );
				do_action( "wplr_remove_media_from_collection", (int)$info->wp_id, (int)$wp_col_id, true );
			}
			$count++;
		}
		// Re-add in the collections with the right order
		foreach ( $lr_ids as $lr_id ) {
			$info = $this->get_sync_info_from_lr_id( $lr_id );
			if ( !empty( $info ) )
				do_action( "wplr_add_media_to_collection", (int)$info->wp_id, (int)$wp_col_id, true );
		}
		return true;
	}

	/*****************************************************************************
		USEFUL FUNCTIONS
	*****************************************************************************/

	function get_upload_root()
	{
		$uploads = wp_upload_dir();
		return $uploads['basedir'];
	}

	// Converts PHP INI size type (e.g. 24M) to int
	function parse_ini_size( $size ) {
		$unit = preg_replace('/[^bkmgtpezy]/i', '', $size);
		$size = preg_replace('/[^0-9\.]/', '', $size);
		if ( $unit )
			return round( $size * pow( 1024, stripos( 'bkmgtpezy', $unit[0] ) ) );
		else
			round( $size );
	}

	// This function should not work even with HVVM
	function b64_to_file( $str ) {

		// From version 1.3.4 (use uploads folder for tmp):
		if ( !file_exists( trailingslashit( $this->get_upload_root() ) . "wplr-tmp" ) )
			mkdir( trailingslashit( $this->get_upload_root() ) . "wplr-tmp" );
		$file = tempnam( trailingslashit( $this->get_upload_root() ) . "wplr-tmp", "wplr_" );

		// Before version 1.3.4:
		//$file = tempnam( sys_get_temp_dir(), "wplr" );

		$ifp = fopen( $file, "wb" );
		fwrite( $ifp, base64_decode( $str ) );
		fclose( $ifp );
		chmod( $file, 0664 );
		return $file;
	}

	/*****************************************************************************
		POST TYPE / COLLECTION RELATED COLUMN
	*****************************************************************************/

	function html_for_collection( $collection_id ) {
		$sync = $this->get_collection( $collection_id );
		$html = "";
		if ( !$sync ) {
			$html .= "<div>" . __( "Unknown", 'wplr-sync' ) . "</div>";
		}
		else {
			$html .= "<div style='color: #006EFF;'>" . __( "Enabled", 'wplr-sync' ) . "</div>";

			if ( !strtotime( $sync->lastsync ) || $sync->lastsync == "0000-00-00 00:00:00" )
				$html .= "<div style='color: #0BF;'><small>" . __( "Never synced.", 'wplr-sync' ) . "</small></div>";
			else {
				if ( date('Ymd') == date('Ymd', strtotime( $sync->lastsync ) ) ) {
					$html .= "<div><small>" .
						sprintf( __( "Synced at %s", 'wplr-sync' ), date("g:ia", strtotime( $sync->lastsync ) ) ) .
						"</small></div>";
				}
				else {
					$html .= "<div><small>" .
						sprintf( __( "Synced at %s", 'wplr-sync' ), date("Y/m/d", strtotime( $sync->lastsync ) ) ) .
						"</small></div>";
				}
			}
			//$html .= "<div><small>LR COL ID: " . $sync->lr_col_id . "</small></div>";
		}
		return $html;
	}

	/*****************************************************************************
		MEDIA LIBRARY COLUMN
	*****************************************************************************/

	function html_for_media( $wpid, $sync = null ) {
		$wpid = $this->wpml_original_id($wpid);
		$html = "";
		if ( !$sync ) {
			$html .= "<div>" . __( "Unknown", 'wplr-sync' ) . "</div>";
			$html .= "<div>
			<small>LR ID:
				<input type='text' class='wplr-sync-lrid-input wplrsync-link-" . $wpid . "'></input>
				<span class='wplr-button' onclick='wplrsync_link($wpid)'>" . __( "Link", 'wplr-sync' ) . "</span>
			</small></div>";
		}
		else {
			if ( $sync->lr_id > 0 ) {
				$html .= "<div style='color: #006EFF;'>" . __( "Enabled", 'wplr-sync' ) . "</div>";

				if ( !strtotime( $sync->lastsync ) || $sync->lastsync == "0000-00-00 00:00:00" )
					$html .= "<div style='color: #0BF;'><small>" . __( "Never synced.", 'wplr-sync' ) . "</small></div>";
				else {
					if ( date('Ymd') == date('Ymd', strtotime( $sync->lastsync ) ) ) {
						$html .= "<div><small>" .
							sprintf( __( "Synced at %s", 'wplr-sync' ), date("g:ia", strtotime( $sync->lastsync ) ) ) .
							"</small></div>";
					}
					else {
						$html .= "<div><small>" .
							sprintf( __( "Synced at %s", 'wplr-sync' ), date("Y/m/d", strtotime( $sync->lastsync ) ) ) .
							"</small></div>";
					}
				}
				$html .= "<div><small>LR ID: " . $sync->lr_id . "</small></div>";
			}
			else if ( $sync->lr_id == 0 ) {
				$html .= "<div style='color: gray;'>" . __( "Ignored", 'wplr-sync' ) . "</div>";
			}
			$html .= "<small><span class='wplr-link-undo' onclick='wplrsync_unlink($sync->lr_id, $wpid)'>" .
				__( "(undo)", 'wplr-sync' ) .
				"</span></small>";
		}
		return $html;
	}

	function wplrsync_unlink() {
		$this->wplrsync_ajax_link_unlink(true);
	}

	function wplrsync_link() {
		$this->wplrsync_ajax_link_unlink();
	}

	function wplrsync_ajax_link_unlink( $is_unlink = false ) {
		if ( !current_user_can('upload_files') ) {
			echo json_encode( array( 'success' => false, 'message' => "You do not have the roles to perform this action." ) );
			die;
		}
		if ( !isset( $_POST['lr_id'] ) || $_POST['lr_id'] == "" || !isset( $_POST['wp_id'] ) || empty( $_POST['wp_id'] ) ) {
			echo json_encode( array( 'success' => false, 'message' => "Some information is missing." ) );
			die;
		}

		$lr_id = intval( $_POST['lr_id'] );
		$wp_id = $this->wpml_original_id( intval( $_POST['wp_id'] ) );

		$sync = null;
		if ( $is_unlink ) {

			if ( $this->unlink_media( $lr_id, $wp_id ) ) {
				echo json_encode( array(
					'success' => true,
					'html' => $this->html_for_media( $wp_id, null )
				) );
			}
			else {
				echo json_encode( array(
					'success' => false,
					'message' => $this->error || "Unknown error."
				) );
			}
		}
		else {
			$sync = $this->link_media( $lr_id, $wp_id );
			if ( $sync ) {
				echo json_encode( array(
					'success' => true,
					'html' => $this->html_for_media( $wp_id, $sync )
				) );
			}
			else {
				echo json_encode( array(
					'success' => false,
					'message' => $this->error || "Unknown error."
				) );
			}
		}
		die();
	}

	function admin_head() {
		echo '
			<style type="text/css">

				.wplr-button {
					background: #3E79BB;
					color: white;
					display: inline;
					padding: 2px 8px;
					border-radius: 7px;
					text-transform: uppercase;
					margin-left: 1px;
				}

				.wplr-button:hover {
					cursor: pointer;
					background: #5D93CF;
				}

				.wplr-link-undo {
					color: #5E5E5E;
				}

				.wplr-link-undo:hover {
					cursor: pointer;
					color: #2ea2cc;
				}

				.wplr-sync-info {
					line-height: 14px;
				}

				.wplr-sync-lrid-input {
					width: 56px;
					font-size: 10px;
					font-weight: bold;
					color: black !important;
				}

			</style>

			<script>

				function wplrsync_handle_response( wp_id, response ) {
					reply = jQuery.parseJSON(response);
					if ( reply.success ) {
						// Remove box (if in WP/LR Dashboard)
						jQuery("#wplr-image-box-" + wp_id).remove();
						// Update row (if in Media Library)
						jQuery(".wplrsync-media-" + wp_id).html(reply.html);
					}
					else {
						alert(reply.message);
					}
				}

				function wplrsync_unlink( lr_id, wp_id ) {
					var data = { action: "wplrsync_unlink", lr_id: lr_id, wp_id: wp_id };
					jQuery.post(ajaxurl, data, function (response) {
						wplrsync_handle_response( wp_id, response );
					});
				}

				function wplrsync_link( wp_id, ignore ) {
					if (!ignore) {
						lr_id = jQuery(".wplrsync-link-" + wp_id).val();
					}
					else {
						lr_id = 0;
					}
					var data = { action: "wplrsync_link", lr_id: lr_id, wp_id: wp_id };
					jQuery.post(ajaxurl, data, function (response) {
						wplrsync_handle_response( wp_id, response );
					});
				}
			</script>
		';
	}

	function manage_media_columns( $cols ) {
		$cols["WPLRSync"] = "LR Sync";
		return $cols;
	}

	function manage_media_custom_column( $column_name, $wpid ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "lrsync";
		if ( $column_name != 'WPLRSync' )
			return;
		$meta = wp_get_attachment_metadata( $wpid );
		if ( !($meta && isset( $meta['width'] ) && isset( $meta['height'] )) ) {
			return;
		}
		$wpid = $this->wpml_original_id( $wpid );
		$sync = $this->get_sync_info( $wpid );
		echo "<div class='wplr-sync-info wplrsync-media-" . $wpid . "'>";
		echo $this->html_for_media( $wpid, $sync );
		echo "</div>";
	}

	/*****************************************************************************
		HELPERS FOR TAXONOMIES MANAGEMENT
		USED BY KEYWORDS (CORE) AND POST TYPES (EXTENSION)
	*****************************************************************************/

	function create_taxonomy( $keywordId, $inKeywordId, $keyword, $taxonomy, $metaKey ) {
		global $wplr;

		$is_term_exists = false;

		$term = get_term_by( 'name', $keyword['name'], $taxonomy );
		if ( !empty( $term ) ) {
			$wplr->set_meta( $metaKey, $keywordId, $term->term_id, true );
			$is_term_exists = true;
		}

		// Create term
		if ( !$is_term_exists ) {
			$parentTermId = null;
			if ( !empty( $inKeywordId ) && is_taxonomy_hierarchical( $taxonomy ) )
				$parentTermId = $wplr->get_meta( $metaKey, $inKeywordId );
			$result = wp_insert_term( $keyword['name'], $taxonomy, $parentTermId ? array( 'parent' => $parentTermId ) : null );
			if ( is_wp_error( $result ) ) {
				error_log( "Issue while creating the keyword " . $keyword['name'] . "." );
				error_log( $result->get_error_message() );
				return;
			}
			$wplr->set_meta( $metaKey, $keywordId, $result['term_id'], true );
		}
	}

  function update_taxonomy( $folderId, $folder, $taxonomy, $metaKey ) {
    global $wplr;
    $termId = $wplr->get_meta( $metaKey, $folderId );
    wp_update_term( $termId, $taxonomy, array( 'name' => $folder['name'] ) );
  }

  // Move the folder (category) under another one.
  // If the folder is empty, then it is the root.
  function move_taxonomy( $folderId, $inFolderId, $taxonomy, $metaKey ) {
    global $wplr;
    $termId = $wplr->get_meta( $metaKey, $folderId );
    $parentTermId = null;
    if ( !empty( $inFolderId ) )
      $parentTermId = $wplr->get_meta( $metaKey, $inFolderId );
    wp_update_term( $termId, $taxonomy, array( 'parent' => $parentTermId ) );
  }

  function remove_taxonomy( $folderId, $taxonomy, $postType, $metaKey ) {
    global $wplr;
    $id = $wplr->get_meta( $metaKey, $folderId );
    $objs = get_objects_in_term( $id, $taxonomy );
    $args = array(
      'post_type' => $postType,
      'tax_query' => array(
        array( 'taxonomy' => $taxonomy, 'field' => 'id', 'terms' => (int)$id )
      )
    );
    $query = new WP_Query( $args );
    if ( $query->found_posts < 1 ) {
      $r = wp_delete_term( $id, $taxonomy );
      if ( is_wp_error( $r ) ) {
        error_log( "Issue while deleting the folder " . $folderId . "." );
        error_log( $r->get_error_message() );
        return;
      }
      $wplr->delete_meta( $metaKey, $folderId );
    }
  }

	// If postMetaKey is null, then we consider the postId is the ID
	// of the post type already (good for Media in Keywords)
  function add_taxonomy_to_posttype( $folderId, $collectionId, $taxonomy, $postMetaKey, $termMetaKey ) {
    global $wplr;
    $term = $this->get_term_from_folder( $folderId, $taxonomy, $termMetaKey );
    if ( !empty( $term ) ) {
      $postId = empty( $postMetaKey ) ? $collectionId : $wplr->get_meta( $postMetaKey, $collectionId );
      if ( empty( $postId ) ) {
        error_log( "Cannot find the post for $collectionId (postMetaKey: $postMetaKey)." );
        return;
      }
      $terms = wp_get_post_terms( $postId, $taxonomy, array( 'fields' => 'ids' ) );
      $terms[] = $term->term_id;
      $r = wp_set_post_terms( $postId, $terms, $taxonomy );
      if ( is_wp_error( $r ) )
        error_log( $r->get_error_message() );
    }
  }

	// If postMetaKey is null, then we consider the postId is the ID
	// of the post type already (good for Media in Keywords)
  function remove_taxonomy_from_posttype( $folderId, $collectionId, $taxonomy, $postMetaKey, $termMetaKey ) {
    global $wplr;
    if ( empty( $folderId ) )
      return;
    $postId = empty( $postMetaKey ) ? $collectionId : $wplr->get_meta( $postMetaKey, $collectionId );
    if ( empty( $postId ) ) {
      error_log( "Cannot find the post for $collectionId (postMetaKey: $postMetaKey)." );
      return;
    }
    $folderId = empty( $folderId ) ? null : $wplr->get_meta( $termMetaKey, $folderId );
    if ( empty( $folderId ) ) {
      //error_log( "Cannot find the related term for folder $folderId." );
      return;
    }
    $r = wp_remove_object_terms( (int)$postId, (int)$folderId, $taxonomy );
    if ( is_wp_error( $r ) )
      error_log( $r->get_error_message() );
  }

  function get_term_from_folder( $folderId, $taxonomy, $termMetaKey ) {
    global $wplr;
    if ( empty( $folderId ) )
      return;
    $parentTermId = $wplr->get_meta( $termMetaKey, $folderId );
    if ( empty( $parentTermId ) ) {
      error_log( "Cannot find the term for $folderId." );
      return;
    }
    $term = get_term_by( 'term_id', $parentTermId, $taxonomy );
    if ( empty( $term ) ) {
      error_log( "Cannot find information for the term $parentTermId (folder $folderId)." );
      return;
    }
    return $term;
  }

}

?>
